App = {
	conf: window.Configuration || {},
	roadsheet: null,
	summary: null,

	spinner: document.getElementById('roadsheetSpinner'),
	roadsheetHeader: document.getElementById('roadmap-content-header'),
	roadsheetHeaderText: document.getElementById('roadmap-content-header-text'),

	roadsheetHeaderTextTranslation: {
		fra: 'Feuille de route',
		eng: 'Roadmap',
		spa: 'Hoja de ruta',
		deu: 'Routenplan',
		ita: 'Foglio di viaggio'
	},

	init: function () {
		window.addEventListener('load', function () {
			var networkAvailable;
			if (App.conf.debug) {
				JSE._alert = function () { 
					if (console && console.log) {
						console.log(arguments[1] && arguments[1].toString(), arguments[1] && arguments[1].stack,  arguments);
					}
				};
				JSE.alertMode();
			}

			if (typeof App.conf.networkAvailable === 'undefined') {
				networkAvailable = true;
			} else {
				networkAvailable = !!App.conf.networkAvailable;
			}

			App.addStyle();
			App.renderSummary(App.conf.summary, networkAvailable);
		}, false);
	},

	addStyle: function () {
		App.conf.style = App.conf.style || 'ios';

				var docElt = document.documentElement;
				docElt.className = docElt.className.replace(/(^|\s)ios(\s|$)/g, '$1$2').replace(/(^|\s)android(\s|$)/g, '$1$2');
				docElt.className += App.conf.style === 'android' ? ' android' : ' ios';
	},

	renderSummary: function (summaryData, networkAvailable) {
		if (typeof networkAvailable === 'undefined') {
			networkAvailable = true;
		}
		if (summaryData) {
			if (summaryData.distance && summaryData.distance.unit) {
				App.distanceUnit = summaryData.distance.unit;
			}
			App.vehiculeType = summaryData.vehiculeType;
			App.favorMotorway = summaryData.favorMotorway;

			if (App.summary && App.summary.render) {
				App.summary.render(summaryData, networkAvailable);
			} else {
				JSELaunch('Summary', {lang: App.conf.lang, device: App.conf.device, model: App.conf.model, useExternalLinksInAd: App.conf.useExternalLinksInAd}, {
					onInit: function (summary) {
						App.summary = summary;
						App.summary.render(summaryData, networkAvailable);
						App.renderRoadsheet(App.conf.roadsheet);
					}
				});
			}
		} else {
			App.renderRoadsheet(App.conf.roadsheet);
		}
	},

	renderRoadsheet: function (roadsheetData) {
		var aRoadsheet;
		App.translateRoadsheetHeader();
		App.roadsheetHeader.style.display = 'block';
		App.startLoading();
		
		if (roadsheetData && roadsheetData.length >= 1) {
			aRoadsheet = roadsheetData[0];
		} else if (roadsheetData && roadsheetData.content && roadsheetData.content.length >= 1) {
			aRoadsheet = roadsheetData.content[0];
		}

		if (aRoadsheet) {
			if (App.roadsheet && App.roadsheet.render) {
				App.roadsheet.render(aRoadsheet);
			} else {
				JSELaunch('Roadsheet', {
					startLoading: App.startLoading,
					stopLoading: App.stopLoading,
          distanceUnit: App.distanceUnit,
					favorMotorway: App.favorMotorway,
					vehiculeType: App.vehiculeType,
					adConfig: App.conf.adConfig,
					lang: App.conf.lang,
					mobileStaticBaseUrl: App.conf.mobileStaticBaseUrl,
					useExternalLinksInAd: App.conf.useExternalLinksInAd
				}, {
					onInit: function (roadsheet) {
						App.roadsheet = roadsheet;
						App.roadsheet.render(aRoadsheet);
					}
				});
			}
		}
	},

	translateRoadsheetHeader: function () {
		var translatedHeaderText = App.roadsheetHeaderTextTranslation[App.conf.lang || 'fra'] || 'Feuille de route';
		if (App.roadsheetHeaderText && translatedHeaderText) {
			App.roadsheetHeaderText.innerHTML = translatedHeaderText;
		}
	},

	startLoading: function () {
		if (App.conf.neverShowSpinner !== true) {
			App.spinner.style.display = 'block';
		}
	},
	stopLoading: function () {
		App.spinner.style.display = 'none';
	}
};

App.init();
