JSEPackage("Roadsheet");

JSEImportTrad("ViaMichelin.Api.Trad.Itinerary.RoadSheet");
JSEImportApi("ViaMichelin.Api.Model.Itinerary.RoadSheet");
JSEImportApi("ViaMichelin.Api.Itinerary.RoadSheetView");

Roadsheet = function () {
	// Load the css
	var urls = ViaMichelin.Api.Util.generateUrlCss("ApiRoadSheet", "", {ie6:true});
	ViaMichelin.Api.Util.loadCss("ApiRoadSheet", urls);

	this.container = document.getElementById('roadmap-content-text');
};

Roadsheet.prototype = {
	init: function (conf) {
		this.startLoading = conf.startLoading;
		this.stopLoading = conf.stopLoading;
		this.distanceUnit = conf.distanceUnit || 'km';
		this.favorMotorway = conf.favorMotorway || false;
		this.vehiculeType = conf.vehiculeType || 0;
		this.adConfig = conf.adConfig;
		this.lang = conf.lang;
		this.mobileStaticBaseUrl = conf.mobileStaticBaseUrl;
		this.useExternalLinksInAd = conf.useExternalLinksInAd;

		if (this.useExternalLinksInAd) {
			this.addExternalToSecuriteRoutiereLinks();
		}
	},

	clean: function () {
		if (this.roadsheet && typeof this.roadsheet.release === 'function') {
			this.roadsheet.release();
		}
		if (this.roadsheetview && typeof this.roadsheetview.release === 'function') {
			this.roadsheetview.release();
		}
		this.container.innerHTML = "";
	},

	render: function (roadsheetObject) {
		this.startLoading();
		this.clean();

		this.prepareSecuriteRoutiereAd(this.favorMotorway, this.vehiculeType);

		this.roadsheet = new ViaMichelin.Api.Model.Itinerary.RoadSheet(
			ViaMichelin.Api.Trad.Itinerary.RoadSheet, null, this.distanceUnit, roadsheetObject
		);

		this.roadsheetview = new ViaMichelin.Api.Itinerary.RoadSheetView(this.roadsheet);
		this.roadsheetview.addTo({
			container: this.container,
			withSchema: false,
			mode: 'NORMAL',
			display: 'WEB',
			version: 2
		});

		this.stopLoading();
	},

	prepareSecuriteRoutiereAd: function (shouldFavorMotorway, vehiculeType) {
		var element, securiteRoutiereTwohours, config;
		var mobileStaticBaseUrl = this.mobileStaticBaseUrl;

		var divContaining = function (html) {
				var div = document.createElement('div');
				div.innerHTML = html;
				return div;
		};

		var fillBaseUrl = function (string) {
			return string.replace(/\{\{baseUrl\}\}/g, mobileStaticBaseUrl);
		};

		var append = function (element, conf, type, component) {
			var componentHtml = conf[type] && conf[type][component];
			if (componentHtml) {
				componentHtml = fillBaseUrl(componentHtml);
				element.appendChild(divContaining(componentHtml));
			}
		};

		// Change message for securite routiere advertisement
		if (this.lang === "fra" && !!this.adConfig) {
			config = shouldFavorMotorway ? this.adConfig.favorMotorway : this.adConfig.dontFavorMotorway;
			element = $_id('roadmap-content-ad');
			while (element.firstChild) {
				element.removeChild(element.firstChild);
			}

			['style', 'hea' + 'der', 'belt'].forEach(function (component) {
				append(element, config, vehiculeType.toString(), component);
			});

			securiteRoutiereTwohours = config[vehiculeType.toString()] && config[vehiculeType.toString()].twohours;
			if (securiteRoutiereTwohours) {
				ViaMichelin.Api.Trad.Itinerary.RoadSheet.lab_security = fillBaseUrl(securiteRoutiereTwohours);
			} else {
				ViaMichelin.Api.Trad.Itinerary.RoadSheet.lab_security = "";
			}
		}
	},

	addExternalToSecuriteRoutiereLinks: function () {
		var that = this;
		var replacer = function (match) {
			var replacement = match.replace(/(https?:\/\/)/, 'external-$1');
			return replacement;
		};
		[ 'favorMotorway', 'dontFavorMotorway' ].forEach(function (motorWayPref) {
			[ '0', '2', '3', '4' ].forEach(function (vehiculeType) {
				if (that.adConfig[motorWayPref][vehiculeType]) {
					['style', 'hea' + 'der', 'belt', 'twohours'].forEach(function (component) {
						if (that.adConfig[motorWayPref][vehiculeType][component]) {
							var theVehiculeType = that.adConfig[motorWayPref][vehiculeType];
							theVehiculeType[component] = theVehiculeType[component].replace(/href=\\?(:?'|")https?:\/\//g, replacer);
						}
					});
				}
			});
		});
	}
};
