JSEPackage("ViaMichelin.Api.Model.GeoCoordinates");

/**
 * Objet permettant de gérer un point avec une longitude et une latitude
 * 
 * @class ViaMichelin.Api.Model.GeoCoordinates
 * @param {String} longitude
 * @param {String} latitude
 * @constructor
 */
ViaMichelin.Api.Model.GeoCoordinates = function(longitude, latitude) {
	if ($isDefined(longitude)) this.lon = longitude;
	if ($isDefined(latitude))	 this.lat = latitude;
	this.longitude = this.lon;
	this.latitude = this.lat;
};

ViaMichelin.Api.Model.GeoCoordinates.prototype = {
	lon : null,
	lat : null,
	/**
	 * Indique si les coordonnées sont valides
	 */
	areValid : function() {
		if (!$isDefined(this.lon) || !$isDefined(this.lat) 
				|| ( (this.lon == 0.0) && (this.lat == 0.0))) {
			return false;
		}
		/* La longitude sur Terre peut varier de -180 à 180
		 * La latitude sur Terre peut varier de -90 à 90
		 * La latitude sur les cartes ViaMichelin peut varier de -83 à 84
		 */
		return (this.lon >= -180.0) && (this.lon <= 180.0)
			&& (this.lat >= -83.0) && (this.lat <= 84.0);
	},
	/*
	 * Computes distances between 2 geo-coordinates
	 */
	distanceFrom : function(pointB){
		//http://www.movable-type.co.uk/scripts/gis-faq-5.1.html
		var lat1 = this.fDeg2rad(this.lat);
		var lon1 = this.fDeg2rad(this.lon);
		var lat2 = this.fDeg2rad(pointB.lat);
		var lon2 = this.fDeg2rad(pointB.lon);
		var dlat = lat2 - lat1;
		var dlon = lon2 - lon1;
		var a  = Math.pow(Math.sin(dlat/2),2) + Math.cos(lat1) * Math.cos(lat2) * Math.pow(Math.sin(dlon/2),2);
		return 2 * Math.atan2(Math.sqrt(a),Math.sqrt(1-a)) * 6367000;		
	},
	fDeg2rad : function(deg) {
		return deg * Math.PI/180;
	},
	toString : function() {
		return "[ViaMichelin.Api.Model.GeoCoordinates] {" + this.lon + "," + this.lat +"}";
	}
};