JSEPackage("ViaMichelin.Api.Model.Poi");

JSEImportApi("ViaMichelin.Api.Model.GeoCoordinates");
JSEImportApi("ViaMichelin.Api.Map.Layer");

/**
 * @class ViaMichelin.Api.Model.Poi
 * @constructor
 */
ViaMichelin.Api.Model.Poi = function() {
	this.prepare();
};

ViaMichelin.Api.Model.Poi.IconDesc = {
	//path : ViaMichelin.Api.Constants.System.ApiLang + "/map_icon/mapiconpoi",
	path : "mapiconpoi",
	ext : ".png"
};

ViaMichelin.Api.Model.Poi.prototype = {
	/** @type String */
	id : null,
	/** @type String */
	/* lg : null, eciste mais mis en commentaire pour ne pas faire planter l'obfuscation B2C */
	/** @type String */
	name : null,
	/** @type String */
	translatedName : null,
	/** @type String */
	encodedName : null,
	location : {
		/** @type ViaMichelin.Api.Model.GeoCoordinates */
		coords : null,
		/** @type String */
		formattedAddressLine : null,
		/** @type String */
		additionalAddressLine : null,
		/** @type String */
		formattedCityLine : null,
		/** @type String */
		streetNumber : null,
		/** @type String */
		streetLabel : null,
		/** @type String */
		city : null,
		/** @type String */
		postalCode : null,
		/** @type String */
		reflexId : null,
		/** @type String */
		area : null,
		/** @type String */
		countryISO : null,
		/** @type String */
		countryOfficial : null,
		/** @type String */
		countryLabel : null
	},
	datasheet : {
		/** @type Number */
		iconId : null,
		/** @type String */
		geolvl : null,
		tel : {
			/** @type String */
			number : null,
			/** @type String */
			intNumber : null
		},
		fax : {
			/** @type String */
			number : null,
			/** @type String */
			intNumber : null
		},
		/** @type String */
		email : null,
		/** @type String */
		web : null,
		/** @type String */
		url : null,
		reviews : {
			count : null,
			avgRating : null
		},
		/** @type String */
		dataPath : null,
		/** @type String */
		imgPath : null,
		/** @type String */
		mapIconPath : null,
		/**
		 * Exemple : [{idx, value}]
		 * @type Array<Object>
		 */
		metanumList : [],
		/**
		 * Exemple : [{idx, value}]
		 * @type Array<Object> 
		 */
		metastringList : [],
		/** 
		 * Exemple: [{idx, lib, visible, value}] 
		 * @type Array<Object> 
		 */
		descList : [],
		/** 
		 * Exemple : [{idx, value}] 
		 * @type Array<Object>
		 */
		photoList : [],
		/**
		 * Exemple : [{idx, value}] 
		 * @type Array<Object>
		 */
		categoryList : [],
		internal : {
			isMichelinSelection : false
		}
	},
	/** type ViaMichelin.Api.Map.Layer */
	layer : null,
	offsetX : 0,
	offsetY : 0,	
	/**
	 * 	@description Return the layer to add on the map
	 *  @return ViaMichelin.Api.Map.Layer
	 *  @see ViaMichelin.Api.Map#addLayer
	 *  */
	getLayer : function(index) {
		if (this.layer == null) {
			if (index == undefined) {
				index = "";
			}
			var html = this._drawPoiMap(index, "green");
			this.layer = new ViaMichelin.Api.Map.Layer(this.location.coords, html, -10, -29);
			this.layer.HTMLElement.title = this.name;
			this.layer.HTMLElement.alt = this.name;
			this.layer.HTMLElement.style.cursor = "pointer";
			this.layer.setExpandLayer(this._drawExpandLayer());
		}
		return this.layer;
	},
	/**
	 * 	@description Release the data and remove the layer from the map if necessary
	 *  */
	release : function() {
		if (this.layer != null) {
			if (this.layer.mapObject != null) {
				this.layer.mapObject.removeLayer(this.layer);
			}
			this.layer.release();
			this.layer = null;
		}
	},
	toString : function(){
		return "[ViaMichelin.Api.Model.Poi]";
	},
	_drawPoiMap: function (index, color) {
		var strHTML = "";
		var strHover = "onMouseOver=\"this.parentNode.style.zIndex=1000;\" onmouseout=\"this.parentNode.style.zIndex=2\"";
		
		var icon;
		if($isDefined(this.iconPath)){
			icon = this.iconPath.replaceAll('{ICON_ID}', this.datasheet.iconId);
		}else if ($isDefined(this.datasheet.mapIconPath) && $isDefined(this.datasheet.iconId)) {
			icon = ViaMichelin.Api.Constants.System.ApiPath 
				+ '/' + this.datasheet.mapIconPath
				+ ViaMichelin.Api.Model.Poi.IconDesc.path
				+ this.datasheet.iconId
				+ ViaMichelin.Api.Model.Poi.IconDesc.ext;			
		} else {
			// Default icon
			icon = ViaMichelin.Api.Constants.System.ApiImgPath + 'all/pastille/' + color + '_big_shaded_blazon.png';
		}
		
		if (navigator.isIE6) {
			//In IE6, image is not displayed if width and height are not specified but we don't know them yet
			//Quick dumb solution: we force 35*35
			strHTML = '<div class="ui-picto-poi" ' + strHover +'><img src="'
			+ ViaMichelin.Api.Constants.System.ApiImgPath + 'all/s.gif"' 
			+ ' width="35" height="35" style="filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\'' + icon + '\', sizingMethod=\'scale\');" />';
		} else {
			strHTML =	'<div class="ui-picto-poi" '+ strHover +'><img src="' + icon + '" />';
		}
		//Start index at 1
		return strHTML + '<div style="top:' + this.offsetY + 'px;left:' +  this.offsetX + 'px">' + (index+1) + '</div></div>';
	},
	_drawExpandLayer : function() {
		var html = "<div class='sPODBubble'>";
		html += "<ul><li>" + this.name + "</li>";
		html += "<li>"	+ this.location.formattedAddressLine + " " + this.location.postalCode + " " + this.location.city + "</li>";
		if (this.datasheet.web) {
			var url = this.datasheet.web.toLowerCase();
			var pos =  url.indexOf("window.open(");
			if(pos != -1){
				url = url.substring(pos+13, url.indexOf(")", pos)-1);
			}
			html +=  '<li><a href="' + (url.substr(0, 4) != "http" ? "http://" : "") + url + '" target="blank">' + url + '</a></li>';
		}
		html += "</ul>";
		
		if (this.datasheet.descList.length != 0) {
			html += "<dl>";
			for (var i = 0; i < this.datasheet.descList.length; i++) {
				var desc = this.datasheet.descList[i];
				
				if (desc.visible && desc.lib != "" && desc.lib != " ") {
					html += "<dt>" + desc.lib + "</dt><dd>" + desc.value + "</dd>";
				}
			}
			html += "</dl>";
		}
		html += "</div>";
		
		return html;
	}
};
ViaMichelin.Api.Model.Poi.prototype["d"+"b"] = null;