JSEPackage("ViaMichelin.Api.UI.Drawing");

ViaMichelin.Api.UI.Drawing = function(container, width, height, options) {
	var value;
	if ($isDefined(container)) {
		if (typeof(container)=="string"){
			this._container = document.getElementById(container);
		} else {
			this._container = container;
		}
	}

	//this._options = ViaMichelin.Api.UI.Drawing.prototype._options;
	for(value in options){
		this._options[value] = options[value];
	}

	this._width	 = parseInt(width);
	this._height = parseInt(height);
	
	this._chooseContainer();
	this._prepareContainer();
};

ViaMichelin.Api.UI.Drawing.prototype = {
	_width 		: 0,
	_height		: 0,
	//_elements	: [],
	_container  : null,
	_canvas		: null,
	_svgDef		: null,
	_options	: {
		rendering	: "auto", //svg, canvas, vml
		className	: null
	},
	
	/**
	 * Dessine un cercle
	 * @param {Array} points
	 * @param {Object} styles
	 */
	drawCircle : function(point, radius, styles){
		var stylesT, i, defaultStylesPath;
		
		defaultStylesPath = {
			"stroke-opacity"	: 1,
			"stroke-width"		: 3,
			"stroke"			: "#000",
			"fill-opacity"		: 1,
			"fill"				: "none"
		};
		
		if(!$isDefined(styles)){
			styles = {};
		}
		
		//minimum 1 point
		if(!$isDefined(point[0]) && !$isDefined(point[1])){
			point = [0,0];
		}
		
		stylesT = this._convertNameStyle(defaultStylesPath);
		styles = this._convertNameStyle(styles);
		
		for(i in styles){
			stylesT[i] = styles[i];
		}
		
		return this["_drawCircle"+this._options.rendering](point, radius, stylesT);
		
	},
	/**
	 * @param {Array} point
	 * @param {int} radius
	 * @param {Object} styles
	 */
	_drawCircleSVG : function(point, radius, styles){
		var SVGCircle = document.createElementNS('http://www.w3.org/2000/svg', 'circle');
		SVGCircle.setAttribute('cx', point[0]);
		SVGCircle.setAttribute('cy', point[1]);
		SVGCircle.setAttribute('r', radius);
		
		for(var i in styles){
			SVGCircle.setAttribute(i, styles[i]);
		}
		
		this._canvas.appendChild(SVGCircle);
	},
	/**
	 * @param {Array} point
	 * @param {int} radius
	 * @param {Object} styles
	 */
	_drawCircleCANVAS : function(point, radius, styles){
		
		
		
	},
	/**
	 * @param {Array} point
	 * @param {int} radius
	 * @param {Object} styles
	 */
	_drawCircleVML : function(point, radius, styles){
		var HTMLNode = document.createElement("v:oval"); 
		HTMLNode.id = "idCircle_"+ this.uniqueIdSuffix;

		HTMLNode.style.position = "absolute";
		HTMLNode.style.left		= point[0] + 'px';
		HTMLNode.style.top		= point[1] + 'px';
		HTMLNode.style.width	= radius*2 + 'px';
		HTMLNode.style.height	= radius*2 + 'px';
		
		var fillNode = document.createElement("v:fill"); 
			fillNode.color		= styles["fillColor"];
			fillNode.opacity	= styles["fill-opacity"];
		HTMLNode.appendChild(fillNode);

		var itiStroke = document.createElement("v:stroke");
			itiStroke.opacity	= styles["stroke-opacity"];
			itiStroke.color		= styles["stroke-color"];
			itiStroke.weight	= styles["stroke-width"] + "px";
		HTMLNode.appendChild(itiStroke);

		this._canvas.appendChild(HTMLNode);
	},
	/**
	 * Dessine une forme
	 * @param points {Array} liste des points : ex : [[x1,y1],[x2,y2]..] / default [[0,0]]
	 * @param styles (Object) liste des styles : ex : {strokeColor:red,....} / default:{}
	 * @return (DOM Element or Object) si VML or SVG on retourne le path, si CANVAS on retourne le context
	 */
	drawPath : function(points, styles){
		var stylesT, i, defaultStylesPath;
		
		defaultStylesPath = {
			"stroke-opacity"	: 1,
			"stroke-width"		: 3,
			"stroke"			: "#000",
			"fill-opacity"		: 1,
			"fill"				: "none",
			"stroke-linecap"	: "butt",
			"stroke-linejoin"	: "round"
			};
		
		if(!$isDefined(styles)){
			styles = {};
		}
		
		//minimum 1 point
		if(!$isDefined(points[0][0]) && !$isDefined(points[0][1])){
			points = [[0,0]];
		}
		
		stylesT = this._convertNameStyle(defaultStylesPath);
		styles = this._convertNameStyle(styles);
		
		for(i in styles){
			stylesT[i] = styles[i];
		}
		
		return this["_drawPath"+this._options.rendering](points, stylesT);
	},
	
	/**
	 * Dessine une forme en VML
	 * @param {Array} points
	 * @param {Object} styles
	 */
	_drawPathVML : function(points, styles){
		var strPoints = "", len = points.length;
		for (var i = 0; i<len; i++) {
			var point = points[i];
			strPoints += point[0] + "px," + point[1] + "px";
			if (i < len - 1) strPoints += ",";
		}
		
		var HTMLNode = document.createElement("v:polyline");
			HTMLNode.style.position = "absolute";
			HTMLNode.style.cursor = "pointer";
			HTMLNode.points = strPoints;
		
		if ($isDefined(styles["fill"]) && styles["fill"] != "none"){
			var fillNode = document.createElement("v:fill"); 
			fillNode.opacity = styles["fill-opacity"];
			fillNode.color = styles["fill"];
			HTMLNode.appendChild(fillNode);
			HTMLNode.filled = "True";	
		} else {
			HTMLNode.filled = "False";
		}
		
		var itiStroke = document.createElement("v:stroke");
		if($isDefined(styles["stroke"]) && styles["stroke"] != "none"){
			itiStroke.opacity	= styles["stroke-opacity"];
			itiStroke.color		= styles["stroke"];
			itiStroke.weight	= styles["stroke-width"] +"px";
			itiStroke.on		= "True";
			
			if ($isDefined(styles["stroke-linejoin"])){
				itiStroke.joinstyle = styles["stroke-linejoin"];
			}
			
			if ($isDefined(styles["stroke-dasharray"]))
				itiStroke.dashstyle = styles["stroke-dasharray"];
		} else {
			itiStroke.on = "False";
		}
		HTMLNode.appendChild(itiStroke);
		
		this._canvas.appendChild(HTMLNode);
	},
	/**
	 * Dessine une forme en SVG
	 * @param {Array} points
	 * @param {Object} styles
	 */
	_drawPathSVG : function(points, styles){
		var strPoints, i, point, nbsPts, SVGPath;
		
		nbsPts = points.length;
		strPoints = "M " + points[0][0]  + " " + (points[0][1]);
		
		for (var i = 1; i < nbsPts; i++) {	
			point = points[i];
			strPoints += " L " + point[0]  + ' ' + point[1];
		}
		
		SVGPath = document.createElementNS('http://www.w3.org/2000/svg', 'path');
		
		SVGPath.style.cursor = 'pointer';
		
		for(i in styles){
			SVGPath.setAttribute(i, styles[i]);
			SVGPath.setAttribute('d', strPoints);
		}
		
		this._canvas.appendChild(SVGPath);
		
		return SVGPath;
	},	
	/**
	 * Dessine une forme en CANVAS
	 * @param {Array} points
	 * @param {Object} styles
	 */
	_drawPathCANVAS : function(points, styles){
		var context, i, nbsPoints, nbsPts;
		
		nbsPts = points.length;
		
		context = this._canvas.getContext('2d');
		context.beginPath();

		for(i in styles){
			context[i] = styles[i];
			}

		context.moveTo(points[0][0], points[0][1]);

		for (i = 0; i < nbsPts; i ++) {
			context.lineTo(points[i][0], points[i][1]);
	    }
	
		context.stroke();
		if($isDefined(styles["fillStyle"]) &&  styles["fillStyle"]!="none"){
			context.fill();
		}
		
		return context;
	},
	/**
	 * Dessine une flêche
	 * @param points {Array} liste des points composant le tronc de la flèche : ex : [[x1,y1],[x2,y2]..] / default [[0,0]]
	 * @param stylesTrunk (Object) liste des styles du tronc de la flèche : ex : {strokeColor:red,....} / default:{}
	 * @param stylesHead (Object) liste des styles de la tête de la flèche : ex : {strokeColor:red,....} / default:{}
	 
	 * @return (DOM Element or Object) si VML or SVG on retourne le path, si CANVAS on retourne le context
	 */
	drawArrow : function(points, stylesTrunk, stylesHead){
		var stylesT, stylesH, i, defaultStylesTrunk, defaultStylesHead;
		
		defaultStylesTrunk = {
			"stroke-opacity"	: 1,
			"stroke-width"		: 3,
			"stroke"			: "#000",
			"fill-opacity"		: 1,
			"fill"				: "none",
			"stroke-linecap"	: "butt",
			"stroke-linejoin"	: "round"
		};
			
		defaultStylesHead = {
			"stroke-opacity"	: 1,
			"stroke-width"		: 3,
			"stroke"			: "#000",
			"fill-opacity"		: 1,
			"fill"				: "none",
			"stroke-linecap"	: "butt",
			"stroke-linejoin"	: "miter"	
		};
		
		if(!$isDefined(stylesTrunk)){
			stylesTrunk = {};
		}
		
		//minimum 2 points
		if(!$isDefined(points[0][0]) && !$isDefined(points[0][1]) && !$isDefined(points[1][0]) && !$isDefined(points[1][1])){
			points = [[0,0][10,10]];
		}
			
		stylesT = this._convertNameStyle(defaultStylesTrunk);
		stylesH = this._convertNameStyle(defaultStylesHead);
		stylesTrunk = this._convertNameStyle(stylesTrunk);
		stylesHead = this._convertNameStyle(stylesHead);
		
		//surcharge default
		for(i in stylesTrunk){
			stylesT[i] = stylesTrunk[i];
		}
		
		for(i in stylesHead){
			stylesH[i] = stylesHead[i];
		}
		
		return this["_drawArrow"+this._options.rendering](points, stylesT, stylesH);
	},
	/**
	 * Dessine une flèche en SVG
	 * @param {Array} points
	 * @param {Object} stylesTrunk
	 * @param {Object} stylesHead
	 */
	_drawArrowSVG : function(points, stylesTrunk, stylesHead){
		var SVGPath, marker, polyline, i, defs;
		
		SVGPath = this._drawPathSVG(points, stylesTrunk);
		
		if(!$_id(stylesHead.id)){//on verifie si la def est dans le DOM via l'ID
			marker = document.createElementNS('http://www.w3.org/2000/svg','marker');
			marker.setAttribute('id',stylesHead.id);
			marker.setAttribute('refX',2);
			marker.setAttribute('refY',5);
			marker.setAttribute('viewBox','0 0 10 10');
			marker.setAttribute('markerUnits','strokeWidth');
			marker.setAttribute('orient','auto');
			
			polyline = document.createElementNS('http://www.w3.org/2000/svg','polyline');
			
			//type de fleche
			if(!$isDefined(stylesHead.type)){//default
				polyline.setAttribute('points','0,0 0,10 5,5 0,0');
			}
			else if(stylesHead.type == 1 || stylesHead.type == 2){
				polyline.setAttribute('points','0,0 0,10 5,5 0,0');
			}
			else if(stylesHead.type == 3){
				polyline.setAttribute('points','0,2 8,5 0,8');
			}
			delete stylesHead.type;
			
			for (i in stylesHead) {
				if (i!="polyline") marker.setAttribute(i,stylesHead[i]);
			}
			
			for(var item in stylesHead.polyline){
				polyline.setAttribute(item, stylesHead.polyline[item]);
			}
			
			marker.appendChild(polyline);
			defs = this._canvas.getElementsByTagName("defs");
			
			if(defs.length>0){
				defs[0].appendChild(marker);
			}
		}
				
		SVGPath.setAttribute('marker-end','url(#'+stylesHead.id+')');

		return SVGPath;
	},
	/**
	 * Dessine une flèche en Canvas
	 */
	_drawArrowCANVAS : function(points, stylesTrunk, stylesHead){
		var SVGPath, marker, polyline, i, defs, ptab, pt0, pt1, pt2, points, height, sommet, context, twoLastPoint, nbsPoint, width, angle, type;
		
		context = this._drawPathCANVAS(points, stylesTrunk);
		
		type = stylesHead.type;
		delete stylesHead.type;
		
		function division(a, b){
			if(b == 0){
				return a/(b+0.000001);//Number.MAX_VALUE
			}
			return a/b;
		}
		
		function heightArrow(width, angle) {
			return Math.tan(angle) * width;
		}
		
		function rotate(center, point, angle) {
			return [
				(point[0] - center[0]) * Math.cos(angle) - (point[1] - center[1]) * Math.sin(angle) + center[0],
				(point[0] - center[0]) * Math.sin(angle) + (point[1] - center[1]) * Math.cos(angle) + center[1]
			];
		}

		function droite(point1, point2) {
			var a, b;
					
			a = division( (point1[1] - point2[1]) , (point1[0] - point2[0]));
			b = point1[1] - a * point1[0];
			return {
				a: a,
				b: b
			};
		}

		function point(ptab, point1, point2, width) {
			var x, y, ptAB, ptAC, a, b, ptxa, ptxb, ptya, ptyb;
			a = ptab.a;
			b = ptab.b;
			ptxa = point1[0];
			ptxb = point2[0];
			ptya = point1[1];
			ptyb = point2[1];
			ptAB = Math.sqrt((ptxa - ptxb) * (ptxa - ptxb) + (ptya - ptyb) * (ptya - ptyb));
			ptAC = width;
			x = division((-ptxa * ptxb + ptxa * ptxa + b * ptyb - ptya * ptyb - ptya * b + ptya * ptya - ptAB * ptAC), (-ptxb + ptxa - a * ptyb + a * ptya));
			y = a * x + b;
			return [x, y];
		}
			
		//calcul fleche, à optimiser peut être
		nbsPoint = points.length
		//on oriente la fleche sur les deux dernier points
		
		//paramètre de la fleche en canvas
		
		if(!$isDefined(type)){//default
			angle = Math.PI / 4;//angles de la pointe droite ou gauche
			width = stylesTrunk["lineWidth"]*1.5;//taille de la moitié de la base
		}
		else if(type == 1){
			angle = Math.PI / 4;
			width = stylesTrunk["lineWidth"]*0.9;
		}
		else if(type == 2){
			angle = Math.PI / 4;
			width = stylesTrunk["lineWidth"]*1.5;
		}
		else if(type == 3){
			angle = Math.PI / 2.7;
			width = stylesTrunk["lineWidth"]*1.1;
		}

		//Ces valeurs pourraient être passé en paramètres mais il faut faire une correspondance SVG
		
		//la flêche est orienté suivant les deux derniers points
		twoLastPoint = [
			[points[nbsPoint - 1][0],points[nbsPoint - 1][1]],
			[points[nbsPoint - 2][0],points[nbsPoint - 2][1]]
		];
		
		//les deux derniers points doivent être différents
		if( 
			!((points[nbsPoint - 1][0] == points[nbsPoint - 2][0]) && 
			(points[nbsPoint - 1][1] == points[nbsPoint - 2][1]))
			)
		{
			ptab = droite(twoLastPoint[0], twoLastPoint[1]);
			pt0 = point(ptab, twoLastPoint[0], twoLastPoint[1], width);
			pt1 = rotate(twoLastPoint[0], pt0, Math.PI / 2);
			pt2 = rotate(twoLastPoint[0], pt0, -Math.PI / 2);
			points = [pt1, pt2];//coord droite gauche
			height = heightArrow(width, angle);
			//coord sommet
			sommet = rotate(twoLastPoint[0], point(ptab, twoLastPoint[0], twoLastPoint[1], height), Math.PI);
			//fin calcul
			context.beginPath();
			
			for(i in stylesHead){
				context[i] = stylesHead[i];
			}
	
			context.moveTo(points[0][0], points[0][1]);	
			
			if (!isNaN(sommet[0]) && !isNaN(sommet[1])) {
				
				if(!$isDefined(type) || type == 1 || type==2){//default
					context.moveTo(twoLastPoint[0][0], twoLastPoint[0][1]);
					context.lineTo(points[0][0], points[0][1]);
					context.lineTo(sommet[0], sommet[1]);
					context.lineTo(points[1][0], points[1][1]);
					context.lineTo(twoLastPoint[0][0], twoLastPoint[0][1]);
					context.stroke();
					
					if($isDefined(stylesHead["fillStyle"]) && stylesHead["fillStyle"]!="none"){
						context.fill();
					}
				}
				else if(type == 3){
					context.moveTo(points[1][0], points[1][1]);
					context.lineTo(sommet[0], sommet[1]);
					context.lineTo(points[0][0], points[0][1]);
					context.stroke();
				}
			}
		}
		return context;
	},
	/**
	 * Dessine une flèche en VML
	 */
	_drawArrowVML : function(points, stylesTrunk, stylesHead){
		var angle, width, height, type, ptab, pnt, twoLastPoint, pt1, pt2;
		
		//calcul fleche, à optimiser peut être
		nbsPoint = points.length
		//on oriente la fleche sur les deux dernier points
		
		//paramètre de la fleche en vml
		type = stylesHead.type;
		delete stylesHead.type;
		
		if(!$isDefined(type)){//default
			angle = Math.PI / 4;//angles de la pointe droite ou gauche
			width = stylesTrunk["stroke-width"]*1.5;//taille de la moitié de la base
		}
		else if(type == 1){
			angle = Math.PI / 4;
			width = stylesTrunk["stroke-width"]*0.9;
		}
		else if(type == 2){
			angle = Math.PI / 4;
			width = stylesTrunk["stroke-width"]*1.5;
		}
		else if(type == 3){
			angle = Math.PI / 2.7;
			width = stylesTrunk["stroke-width"]*1.1;
		}
	
		this._drawPathVML(points, stylesTrunk);
		
		//Ces valeurs pourraient être passé en paramètres mais il faut faire une correspondance SVG
		
		//la flêche est orienté suivant les deux derniers points
		twoLastPoint = [
			[points[nbsPoint - 1][0],points[nbsPoint - 1][1]],
			[points[nbsPoint - 2][0],points[nbsPoint - 2][1]]
		];
		
		//les deux derniers points doivent être différents
		if(	!((points[nbsPoint - 1][0] == points[nbsPoint - 2][0]) && 
			(points[nbsPoint - 1][1] == points[nbsPoint - 2][1]))) {
		
			ptab = this._droite(twoLastPoint[0], twoLastPoint[1]);
			pnt = this._point(ptab, twoLastPoint[0], twoLastPoint[1], width);
			pt1 = this._rotate(twoLastPoint[0], pnt, Math.PI / 2);
			pt2 = this._rotate(twoLastPoint[0], pnt, -Math.PI / 2);
			points = [pt1, pt2];//coord droite gauche
			height = this._heightArrow(width, angle);
			
			//coord sommet
			sommet = this._rotate(twoLastPoint[0], this._point(ptab, twoLastPoint[0], twoLastPoint[1], height), Math.PI);
			//fin calcul
			
			if (!isNaN(sommet[0]) && !isNaN(sommet[1])) {
				if (!$isDefined(type) || type == 1 || type==2) {//default
					stylesHead["stroke"] = stylesHead["fill"];
					this._drawPathVML([
						[twoLastPoint[0][0], twoLastPoint[0][1]],
						[points[0][0], points[0][1]],
						[sommet[0], sommet[1]],
						[points[1][0], points[1][1]],
						[twoLastPoint[0][0], twoLastPoint[0][1]]
					], stylesHead);
					
				} else if(type == 3) {
					stylesHead["fill"] = "none";
					this._drawPathVML([
						[points[0][0], points[0][1]],
						[sommet[0], sommet[1]],
						[points[1][0], points[1][1]]
					], stylesHead);
				}
			}
		}
	},
	/**
	 * Converti les noms d'attributs de style suivant le mode de rendu, les nom d'attribut
	 * @param styles (Object) liste des styles
	 * @return (Object) liste des styles transformées
	 */
	_convertNameStyle : function(styles){
		var listeStyles, i, stylesT;
		
		stylesT = {};
		
		listeStyles = {
			"stroke": {CANVAS: "strokeStyle", VML: "stroke", SVG: "stroke" },//à compléter
			"fill": {CANVAS: "fillStyle", VML: "fill", SVG: "fill" },
			"stroke-width": {CANVAS: "lineWidth", VML: "stroke-width", SVG: "stroke-width" },
			"stroke-linejoin": {CANVAS: "lineJoin", VML: "stroke-linejoin", SVG: "stroke-linejoin"},
			"stroke-linecap": {CANVAS: "lineCap", VML: "stroke-linecap", SVG: "stroke-linecap" },
			"type": {CANVAS: "type", VML: "type", SVG: "type" }
			//à compléter
		};
		
		//CANVAS
		//pour le CANVAS, si le style n'existe pas, on obtient une erreur qui a pour conséquence un non affichage de la forme
		//si le style n'est pas dans la liste des styles ci dessus on le supprime
		//FIN CANVAS
		for(i in styles){
		
			if(!$isDefined(listeStyles[i])){//existe pas
				if(this._options.rendering!="CANVAS"){
					stylesT[i] = styles[i];
				}
			}
			else{
				if(!$isDefined(listeStyles[i][this._options.rendering])){
					//le nom correspond à la clef, c'est le bon nom de style, on laisse
					stylesT[i] = styles[i];
				}
				else{
					//mauvais style appliqué on change
					stylesT[listeStyles[i][this._options.rendering]] = styles[i];
				}
			}
		}
		return stylesT;
	},
	/**
	 * Choix du container
	 */
	_chooseContainer : function(){
		var rendering = this._options.rendering.toUpperCase();
			
		if(rendering == "AUTO"){//automatique choose
			if (navigator.isIE) {
				this._options.rendering = "VML";
				this._canvas = this._createContainerVML();	
			} else {
				this._options.rendering = "SVG";
				this._canvas = this._createContainerSVG();
			}
		}
		else if(rendering == "VML" || rendering == "SVG" || rendering == "CANVAS"){
			this._options.rendering = this._options.rendering.toUpperCase();
			this._canvas = this["_createContainer"+rendering]();
		}
		
		if ($isDefined(this._container)){
			this._container.appendChild(this._canvas);
		}
	},
	/**
	 * Efface le schema
	 */
	clear : function(){
		var rendering = this._options.rendering.toUpperCase();
			
		if(rendering == "AUTO"){//automatique choose
			if (navigator.isIE) {
				this["_clearContainerVML"]();
			} else {
				this["_clearContainerSVG"]();
			}
		}
		else if(rendering == "VML" || rendering == "SVG" || rendering == "CANVAS"){
			this["_clearContainer"+rendering]();
		}
	},
	_clearContainerCANVAS : function() {
		
		
	},
	_clearContainerSVG : function() {
		while (this._canvas.firstChild) {
			this._canvas.removeChild(this._canvas.firstChild);
		}
		this._prepareContainer();
		//this._canvas = this._createContainerSVG();
	},
	_clearContainerVML : function() {
		while (this._canvas.firstChild) {
			this._canvas.removeChild(this._canvas.firstChild);
		}
		//this._createContainerVML();
	},
	_createContainerVML : function(){
		var container;
		
		container = document.createElement("div");
		container.style.width =  this._width+"px";
		container.style.height = this._height+"";
		
		if($isDefined(this._options) && $isDefined(this._options.className)){
			container.className = this._options.className;
		}
		
		return container;
	},
	_createContainerSVG : function (){
		var container;
		
		container = document.createElementNS("http://www.w3.org/2000/svg", "svg");
		container.style.width  = this._width+"px";
		container.style.height = this._height+"px";
		container.setAttribute("width", this._width);
		container.setAttribute("height", this._height);
		container.setAttribute('version', '1.1');
	
		if($isDefined(this._options) && $isDefined(this._options.className)){
			container.setAttribute("class", this._options.className);
		}
			
		return container;
	},
	_createContainerCANVAS : function (){
		var container;
		
		var container = document.createElement("canvas");
			
		container.style.width  = this._width+"px";
		container.style.height = this._height+"px";
		container.setAttribute("width", this._width);
		container.setAttribute("height", this._height);
		
		if($isDefined(this._options) && $isDefined(this._options.className)){
			container.setAttribute("class", this._options.className);
		}
			
		return container;
	},
	/*
	 * Préparation des conteneurs
	 * En SVG on ajoute une balise def qui pourra contenir d'eventuel objet préconfiguré 
	 */
	_prepareContainer : function (){
		var defs;
		
		if(this._options.rendering == "SVG"){
			defs = document.createElementNS('http://www.w3.org/2000/svg','defs');
			this._canvas.appendChild(defs);
		}
	},
	_division : function (a, b){
		if(b == 0){
			return a/(b+0.000001);//Number.MAX_VALUE
		}
		return a/b;
	},
	_heightArrow : function (width, angle) {
		return Math.tan(angle) * width;
	},
	_rotate : function (center, point, angle) {
		return [
			(point[0] - center[0]) * Math.cos(angle) - (point[1] - center[1]) * Math.sin(angle) + center[0],
			(point[0] - center[0]) * Math.sin(angle) + (point[1] - center[1]) * Math.cos(angle) + center[1]
		];
	},
	_droite : function (point1, point2) {
		var a, b;
				
		a = this._division( (point1[1] - point2[1]) , (point1[0] - point2[0]));
		b = point1[1] - a * point1[0];
		return {
			a: a,
			b: b
		};
	},
	_point : function (ptab, point1, point2, width) {
		var ptX, ptY, ptAB, ptAC, ptA, ptB, ptXa, ptXb, ptYa, ptYb;
		ptA = ptab.ptA;
		ptB = ptab.ptB;
		ptXa = point1[0];
		ptXb = point2[0];
		ptYa = point1[1];
		ptYb = point2[1];
		ptAB = Math.sqrt((ptXa - ptXb) * (ptXa - ptXb) + (ptYa - ptYb) * (ptYa - ptYb));
		ptAC = width;
		ptX = this._division((-ptXa * ptXb + ptXa * ptXa + ptB * ptYb - ptYa * ptYb - ptYa * ptB + ptYa * ptYa - ptAB * ptAC), (-ptXb + ptXa - ptA * ptYb + ptA * ptYa));
		ptY = ptA * ptX + ptB;
		return [ptX, ptY];
	}
};