JSEPackage("ViaMichelin.Api.UI.Drawing.ItineraryBuilder");

JSEImportApi("ViaMichelin.Api.UI.Drawing");
JSEImportApi("ViaMichelin.Api.Util");

ViaMichelin.Api.UI.Drawing.ItineraryBuilder = function(container, width, height, orientation, options) {
	var value, defaultOptions;
	
	options = $isDefined(options) ? options : {};
	defaultOptions = {className : "canvasContainer"};
	
	this._orientation = orientation;
	
	for(value in options){
		defaultOptions[value] = options[value];
	}
	
	// on force à 300
	width = 300;
	this.extend(ViaMichelin.Api.UI.Drawing, container, width, height, defaultOptions);
	
	this._createHtmlContainer(width, height);
	//this.loadDivPanneaux();
};

ViaMichelin.Api.UI.Drawing.ItineraryBuilder.prototype = {
	_orientation : "N",
	/**
	 * Contient les panneaux, radars, etc.
	 */
	_infoContainer 	: null,
	_container		: null,
	/**
	 * Dessine les routes adjacentes
	 * @param {Array} value
	 */
	drawAdjacentRoads : function(value){
		var roadType, importance, geoCoord, classes1, classes2, i, nbsPoint, points, len=value.length;
		
		// Convertion des données pour le mode au cap
		var newFormat = [];
		if (this._orientation=="C"){
			for(var i=0;i<len;i++){
				newFormat[i] 	= new Array();
				newFormat[i][0] = this._flipCoords(value[i][0]);
				newFormat[i][1] = value[i][1];
				newFormat[i][2]	= value[i][2];
				newFormat[i][3]	= value[i][3];
			}
		} else newFormat = value;
		
		//2 boucles inévitable dans le cas des tronçons qui se chevauchent
		for(i=0; i<len; i++){
			points 		= newFormat[i][0];
			roadType 	= newFormat[i][1];
			importance 	= newFormat[i][2];
			geoCoord 	= newFormat[i][0];
			classes1 	= this.getCssClass("adjacent",roadType,importance,1,null,null);
			
			this.drawPath(points, this.getStyle(classes1));
		}
		
		for(i=0; i<len; i++){
			points 		= newFormat[i][0];
			roadType 	= newFormat[i][1];
			importance 	= newFormat[i][2];
			geoCoord 	= newFormat[i][0];
			classes2 	= this.getCssClass("adjacent",roadType,importance,2,null,null);
			
			this.drawPath(points, this.getStyle(classes2));
		}
		
		this.drawDirections(newFormat);
	},
	/**
	 * Dessine le tronçon de l'itinéraire
	 * @param {Array} value
	 */
	drawItineraryRoads : function(value){
		var i, len, isAnEnd, initialClass, roadType, isoCountry, geoCoord, importance, classes1, classes2, points;
		len = value.length;
		
		// Convertion des données pour le mode au cap
		var newFormat = [];
		if (this._orientation=="C"){
			for(var i=0;i<len;i++){
				newFormat[i] 	= new Array();
				newFormat[i][0] = this._flipCoords(value[i][0]);
				newFormat[i][1] = value[i][1];
				newFormat[i][2]	= value[i][2];
				newFormat[i][3]	= value[i][3];
			}
		} else newFormat = value;
		
		// 2 boucles inévitable dans le cas des tronçons qui se chevauchent
		for(i=0; i<len; i++){
			points 		= newFormat[i][0];
			isoCountry	= newFormat[i][1];
			roadType	= newFormat[i][2];
			importance	= newFormat[i][3];
			
			isAnEnd = (i == 0 || i + 1 == points.length) ? ' troncon_extremite' : '';
			
			// Si le prochain point n'est pas le même on rajoute "troncon_extremite"
			if($isDefined(newFormat[i+1])){
				var nextPoint = newFormat[i+1][0];
				if(nextPoint[0][0] != points[points.length-1][0] || nextPoint[0][1] != points[points.length-1][1]){
					isAnEnd = ' troncon_extremite';
				}
			}
			if($isDefined(newFormat[i-1])){
				var lastPoint = newFormat[i-1][0];
				if(lastPoint[lastPoint.length-1][0] != points[0][0] || lastPoint[lastPoint.length-1][1] != points[0][1]){
					isAnEnd = ' troncon_extremite';
				}
			}
			
			classes1 = "";
			classes1 += this.getCssClass("itinerary",roadType,importance,1,isoCountry,null);
			classes1 += " iti_commun";
			classes1 += isAnEnd;

			this.drawPath(points, this.getStyle(classes1));
		}
		
		for(i=0; i<len; i++){
			points 		= newFormat[i][0];
			isoCountry	= newFormat[i][1];
			roadType	= newFormat[i][2];
			importance	= newFormat[i][3];
			
			isAnEnd = (i == 0 || i + 1 == points.length) ? ' troncon_extremite' : '';
			// Si le prochain point n'est pas le même on rajoute "troncon_extremite"
			if($isDefined(newFormat[i+1])){
				var nextPoint = newFormat[i+1][0];
				if(nextPoint[0][0] != points[points.length-1][0] || nextPoint[0][1] != points[points.length-1][1]){
					isAnEnd = ' troncon_extremite';
				}
			}
			if($isDefined(newFormat[i-1])){
				var lastPoint = newFormat[i-1][0];
				if(lastPoint[lastPoint.length-1][0] != points[0][0] || lastPoint[lastPoint.length-1][1] != points[0][1]){
					isAnEnd = ' troncon_extremite';
				}
			}
			
			classes2 = "";
			classes2 += this.getCssClass("itinerary",roadType,importance,2,isoCountry,null);
			classes2 += " iti_commun";
			classes2 += isAnEnd;
			
			this.drawPath(points, this.getStyle(classes2));
		}
	},
	/**
	 * Dessine les instructions
	 * @param {Array} value
	 */
	drawInstructions : function(value){
		var i, len, style, len2, classes, styles, size, geoCoord, len=value.length;
		
		// Convertion des données pour le mode au cap
		var newFormat = [];
		if (this._orientation=="C"){
			for(var i=0;i<len;i++){
				newFormat[i] 	= new Array();
				newFormat[i][0] = this._flipCoords(value[i][0]);
				newFormat[i][1] = value[i][1];
				newFormat[i][2]	= value[i][2];
				newFormat[i][3]	= value[i][3];
			}
		} else newFormat = value;
		
		//Pré calcul sur les flêches pour déplacer la pointe de la flêche sur le dernier point. 
		for (i=0 ; i<len ; i++) {
			geoCoord	= newFormat[i][0];
			len2		= geoCoord.length;
			classes		= this.getCssClass("instructions",null,null,1,null,size);
			styles		= this.getStyle(classes);
			
			if(newFormat[i][1]==0) styles["stroke-width"] = 3;
			
			var styleHead = this._getArrowStyles("Triangle_1");
			if(newFormat[i][1]==0) styleHead["stroke-width"] = 2;
			
			// valeur du décalage à effectuer
			var gap = styles["stroke-width"] + styleHead["stroke-width"];
			
			var interval = 0;
			
			if(newFormat[i][1] == 1 && geoCoord.length >= 2){
				var savePoints = [];
				for(var k=0;k<geoCoord.length;k++){
					savePoints[k] = geoCoord[k];
				}
				var interval = 0;
				var lastPoint = savePoints[savePoints.length-1];
				while (interval <= gap && savePoints.length >= 2) {
					interval+= this.vector(savePoints[savePoints.length-1], savePoints[savePoints.length-2]);
					lastPoint = savePoints.pop();
				}
				
				// Si l'intervale calculé et plus grand que la taille de la flêche sinon on ne fait rien
				if(interval > gap){
					var point1 = savePoints[savePoints.length-1];
					var point2 = lastPoint;
					var nGap = interval - gap;
					var distance = this.vector(point1, point2);
					var percent = ( nGap / distance ) * 100;
					
					var nPoint = [
						point1[0] + parseInt(percent*(point2[0] - point1[0])/100),
						point1[1] + parseInt(percent*(point2[1] - point1[1])/100),
					];
					
					savePoints.push(nPoint);
					newFormat[i][0] = savePoints;
				}
			}
		}
		
		for (i=0 ; i<len ; i++) {
			size		= newFormat[i][1];
			geoCoord	= newFormat[i][0];
			len2		= geoCoord.length;
			classes		= this.getCssClass("instructions",null,null,1,null,size);
			styles		= this.getStyle(classes);
			
			if(newFormat[i][1]==0) styles["stroke-width"] = 3;
			
			var styleHead = this._getArrowStyles("Triangle_1");
			if(newFormat[i][1]==0) styleHead["stroke-width"] = 2;
			
			this.drawArrow(geoCoord, styles, styleHead);
		}
		
		for (i=0 ; i<len ; i++) {
			if(newFormat[i][1] == 1){
				size		= newFormat[i][1];
				geoCoord	= newFormat[i][0];
				len2		= geoCoord.length;
				classes		= this.getCssClass("instructions",null,null,2,null,size);
				styles		= this.getStyle(classes);
				
				this.drawArrow(geoCoord, styles, this._getArrowStyles("Triangle_2"));
			}
		}
	},
	vector : function(point1, point2){
		return parseInt(Math.sqrt(Math.pow((point2[0]-point1[0]), 2) + Math.pow((point2[1]-point1[1]),2)));
	},
	/**
	 * Dessine les flèches indiquant les sens des routes
	 * @param {Array} value 
	 */
	drawDirections : function(value){
		var sens, geom_support, len, geoCoord, classes, len2, i;
		
		len = value.length;
		
		for (i=0; i<len; i++) {
			sens = value[i][3];
			geoCoord = value[i][0];
			classes = "sens_unique";

			if (sens != 0) {
				geom_support = this._zeroTo50pctPolyline(geoCoord, sens);
				len2 = geoCoord.length;
				if (len2 >= 2) {//minimum 2 point
					this.drawArrow(geom_support, this.getStyle(classes), this._getArrowStyles("Triangle_3"));					
				}
			}
		}
	},
	drawPuces : function(value, customItems){
		var len = value.length;
		for(var i=0; i<len; i++){
			var checkpoint = value[i], details = value[i].details, j, lenDetails = details.length;
			
			if ($isDefined(checkpoint.schema) && checkpoint.schema.length != 0){
				if (checkpoint.type=="C"){
					// Cas d'un flag
					if (checkpoint.picto=="start"||checkpoint.picto=="step"||checkpoint.picto=="end") { //(isNaN(checkpoint.number)||checkpoint.number=="") {
						this.drawFlag(checkpoint.picto, checkpoint.schema);
					} else if (isNaN(checkpoint.number)||checkpoint.number==""){
						
					} else {
						this.drawIndication("pastille_"+ checkpoint.number, checkpoint.schema);
					}
				} else if (checkpoint.type=="V"){
					var type = checkpoint.type;
					
					if (isNaN(checkpoint.number) || checkpoint.number==""){
						//if (type=="rad") type = checkpoint.type +"_"+ checkpoint.speed;
						//if (type=="vir") type = checkpoint.type +"_"+ checkpoint.num;
					} else {
						type = "pastille_"+ checkpoint.number;
					}
					this.drawIndication(type, checkpoint.schema);
				}
			}
			
			for (j=0; j<lenDetails; j++){
				var detail = details[j], type = detail.type;
				
				if($isDefined(detail.schema) && detail.schema.length != 0){
					if (type=="rad") {
						if (customItems["radar"]) {
							type = detail.type +"_"+ detail.speed;
							this.drawIndication(type, detail.schema);
						}
					} else if (type=="vir") {
						if (customItems["security"]) {
							type = detail.type +"_"+ detail.num;
							this.drawIndication(type, detail.schema);
						}
					} else if (type=="rnu") {
						this.drawRoadLabel(detail.schema, detail.road[0], detail.countryISO, detail.road[1] );
					} else if (type=="fra") {
						this.drawCrossing(detail.model, detail.schema);
					} else if (type=="c"){
						if (customItems["proximity"]) {
							if(detail.model == "tunnel"){
								this.drawIndication(detail.model, detail.schema);
							} else if(detail.model == "bridge"){
								this.drawIndication(detail.model, detail.schema);
							} else {
								this.drawProximity(detail.city, detail.schema);
							}
						}
					} else if (type=="dec"){
						if(customItems["security"])
							this.drawIndication(type, detail.schema);
					}
				}
			}
		}
	},	
	/**
	 * Dessine les drapeaux de départ ou d'arrivé
	 * @param {String} type
	 * @param {Array} schema
	 */
	drawFlag : function(type, schema){
		// Affichage du picto
		var coords = this._flipBox([ schema[1][0], schema[1][1], schema[1][2], schema[1][3] ]);
		var flag = document.createElement("img");
			flag.className = "pictoIndication";
			flag.style.top = coords[1] +"px";
			flag.style.left = coords[0] +"px";
			
		if (type == "start") {
			if(navigator.isIE6){
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif";
				flag.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "/all/iti/greenFlag.png',sizingMethod='scale')";
				flag.width = "16px";
				flag.height = "15px";
				flag.className = "greenFlag";
			} else if(navigator.userAgent.indexOf('Firefox/2') != -1) {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/drapeau_depart.gif";
			} else {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/iti/greenFlag.png";
				flag.width = "16px";
				flag.height = "15px";
			}
		}
		else if (type == "end") {
			if(navigator.isIE6){
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif";
				flag.style = "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "/all/iti/redFlag.png',sizingMethod='scale')"; 
				flag.width = "16px";
				flag.height = "15px";
				flag.className = "greenFlag";
			} else if(navigator.userAgent.indexOf('Firefox/2') != -1) {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/drapeau_arrivee.gif";
			} else {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/iti/redFlag.png";
				flag.width = "16px";
				flag.height = "15px";
			}
		}
		else if (type == "step") {
			if(navigator.isIE6){
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif";
				flag.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ ViaMichelin.Api.Constants.System.ApiImgPath + "/all/iti/blueFlag.png',sizingMethod='scale')"; 
				flag.width = "16px";
				flag.height = "15px";
				flag.className = "greenFlag";
			} else if(navigator.userAgent.indexOf('Firefox/2') != -1) {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/data/itifdr/drapeau_etape.gif";
			} else {
				flag.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/iti/blueFlag.png";
				flag.width = "16px";
				flag.height = "15px";
			}
		}
		
		this._infoContainer.appendChild(flag);
		
	},
	drawPastille : function(type, schema){
		var coords = this._flipBox([ schema[1][0], schema[1][1], schema[1][2], schema[1][3] ]);
		
		var flag = document.createElement('img');
			flag.className = "pictoIndication";
			flag.style.top = coords[1] +"px";
			flag.style.left = coords[0] +"px";
			flag.src = this._getStyleIndication(type).url;
		
		this._infoContainer.appendChild(flag);
	},
	/**
	 * Dessine les panneaux d'indications : radars, virages dangereux, etc. avec un trait pour les situer sur le schema
	 * @param {String} type
	 * @param {Object} schema
	 */
	drawIndication : function(type, schema){
		// Affichage du trait
		var pointsPath = [
			this._flipCoords([ schema[0][0], schema[0][1] ]),
			this._flipCoords([ (schema[1][0]+schema[1][2])/2, (schema[1][1]+schema[1][3])/2 ])
		];
		
		this.drawPath(pointsPath, { "stroke-width": 1 });
		
		// Affichage du mini cercle
		this.drawCircle(this._flipCoords([ schema[0][0], schema[0][1] ]), 1);
		
		// Affichage du picto
		var coords = this._flipBox([ schema[1][0], schema[1][1], schema[1][2], schema[1][3] ]);
		var picto = this._getStyleIndication(type);
		
		var indication = document.createElement('img');
			indication.className	= "pictoIndication";
			indication.style.top	= coords[1] +"px";
			indication.style.left	= coords[0] +"px";
			indication.style.width	= picto.size.width+"px";
			indication.style.height = picto.size.height+"px";
			
		if (navigator.isIE6) {
			indication.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif";
			indication.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ picto.url + "',sizingMethod='scale')";
		} else {
			indication.src = picto.url;
		}
		
		this._infoContainer.appendChild(indication);
	},
	/**
	 * Dessine un picto sur le schema
	 */
	drawPicto : function(url, schema, size){
		// Affichage du trait
		var pointsPath = [
			this._flipCoords([ schema[0][0], schema[0][1] ]),
			this._flipCoords([ (schema[1][0]+schema[1][2])/2, (schema[1][1]+schema[1][3])/2 ])
		];
		
		this.drawPath(pointsPath, { "stroke-width": 1 });
		
		// Affichage du mini cercle
		this.drawCircle(this._flipCoords([ schema[0][0], schema[0][1] ]), 1);
		
		// Affichage du picto
		var coords = this._flipBox([ schema[1][0], schema[1][1], schema[1][2], schema[1][3] ]);
		var indication = document.createElement('img');
			indication.className	= "pictoIndication";
			indication.style.top	= coords[1] +"px";
			indication.style.left	= coords[0] +"px";
		
		if($isNotEmpty(size)){
			indication.style.width	= size.width +"px";
			indication.style.height	= size.height +"px";
		}
		
		if (navigator.isIE6) {
			indication.src = ViaMichelin.Api.Constants.System.ApiImgPath + "all/s.gif";
			indication.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+ url + "',sizingMethod='scale')";
		} else {
			indication.src = url;
		}
		
		this._infoContainer.appendChild(indication);
	},
	
	/**
	 * Dessine des franchissements
	 * 1 : route
	 * 2 : chemin de fer
	 * 3 : rivière
	 * @param {int} type
	 * @param {Object} schema
	 */
	drawCrossing : function(type, schema){
		var style = {};
		if(type==1){
			style = {
				stroke: "#DDD"
			};
		} else if (type==2) {
			style = {
				stroke: "#3D2810"
			};
		} else if (type==3) {
			style = {
				stroke: "#AEB0CC"
			};
		}
		
		var coords = this._flipBox([ schema[0][0], schema[0][1], schema[0][2], schema[0][3] ]);
		
		this.drawPath([[coords[0],coords[1]], [coords[2],coords[3]]], style);
	},
	/**
	 * Dessine les Passages à proximités
	 * @param {String} city
	 * @param {Object} schema
	 */
	drawProximity : function(city, schema){
		var points = schema[1], nbLine= schema[2], coords = this._flipBox([ points[0], points[1], points[2], points[3] ]);
		this.drawCircle(this._flipCoords(schema[0]), 4, { stroke:"#B685E8", fill:"#B685E8" } );
		
		// Découpage du nom de la ville
		// Ex: Villefranche-sur-Saône sur deux ligne
		// Villefranche
		// sur-Saône
		if(nbLine>1){
			var separator = " ";
			if(city.indexOf("-")!=-1) separator = "-";
			var rexp = new RegExp(separator, 'i');
			for(var i=1;i<nbLine;i++){
				city = city.replace(rexp, "<br/>");
			}
		}
		
		var container = document.createElement("div");
			container.className = "cityProximity";
			container.style.top	= coords[1] +"px";
			container.style.left= coords[0] +"px";
			container.innerHTML = city;
		
		this._infoContainer.appendChild(container);
	},
	/**
	 * Dessine les étiquettes des routes ex:A6, E15
	 * @param {Array} value
	 */
	drawRoadsLabel : function(value){
		var len = value.length;
		for(var i=0; i<len; i++){
			var item = value[i];
			this.drawRoadLabel(item[0], item[1], item[2], item[3]);
		}
	},
	/**
	 * Dessine une étiquette de route
	 * @param {Array} points
	 * @param {String} number
	 * @param {String} country
	 * @param {Number} type
	 */
	drawRoadLabel : function(points, number, country, type){
		var coords = this._flipBox([ points[0], points[1], points[2], points[3] ]);
		var width_text = Math.abs(coords[2] - coords[0]);
		var height_text = Math.abs(coords[3] - coords[1]);
		var roadLabel = document.createElement('div');
			roadLabel.style.width = width_text+'px';
			roadLabel.style.height = height_text+'px';
			roadLabel.style.lineHeight = (height_text-2)+'px';
			roadLabel.style.left = coords[0]+'px';
			roadLabel.style.top = coords[1]+"px";
			roadLabel.className = "roadLabel";
		
  		var txt = document.createElement("span");
  			txt.className = "caisson_Default caisson_"+ country +"_rte"+ type +"_carte";
		var txtNode = document.createTextNode(number);
		txt.appendChild(txtNode);
		roadLabel.appendChild(txt);
		this._infoContainer.appendChild(roadLabel);
	},
	/**
	 * Dessine des pointillets
	 * @param {Object} point1
	 * @param {Object} point2
	 * @param {Object} style
	 */
	drawDashPath : function(point1, point2, style){
		var flipPoints = this._flipCoords([ point2 ]);
		var points = [point1, flipPoints[0]];
		ViaMichelin.Api.UI.Drawing.prototype.drawPath.call(this, points, style);
	},
	clear : function() {
		while(this._infoContainer.firstChild)
			this._infoContainer.removeChild(this._infoContainer.firstChild);
		
		ViaMichelin.Api.UI.Drawing.prototype.clear.call(this);
	},
	/**
	 * Retourne les urls des images et les styles d'un type d'indication (radar, pente, etc.)
	 * @param {String} value
	 * @return {Object}
	 */
	_getStyleIndication : function(value){
		var tmp = value.split("_"), type = tmp[0], info = tmp[1], picto, size;
		
		switch(type){
			case "pastille":
				picto = "data/itisteps/e"+ info +".png";
				size = { width:16, height:16 };
				break;
			case "dec" :
				picto= "data/itifdr/pente.png";
				size = { width:18, height:18 };
				break;
			case "vir" :
				picto= "data/itifdr/vir"+info+".png";
				size = { width:18, height:18 };
				break;
			case "rad" :
				picto= "data/itifdr/vitesse"+info+".png";
				size = { width:19, height:19 };
				break;
			case "bridge":
				picto= "data/itifdr/pont.png";
				size = { width:18, height:18 };
				break;
			case "tunnel" :
				picto= "data/itifdr/tunnel.png";
				size = { width:18, height:18 };
				break;
			case "poi-hotel" :
				picto= "pastille/puce_jaune_detour.png";
				size = { width:20, height:29 };
				break;
			case "poi-restaurant" :
				picto= "pastille/puce_rouge_detour.png";
				size = { width:20, height:29 };
				break;
			case "poi-tourism" : 
				picto= "pastille/puce_verte_detour.png";
				size = { width:20, height:29 };
				break;
			default :
				picto = "pastille/puce_white_detour.png";
				size = { width:20, height:29 };
				break;
		}
		url = ViaMichelin.Api.Constants.System.ApiImgPath + "all/"+picto;
		
		return { url:url, size:size };
	},
	/*
	 * Styles des flèches
	 */
	_getArrowStyles : function(value){
		var styles = {};
		if(value == "Triangle_1"){
			styles = {
				"id"			: "Triangle_1",//obligatoire
				"fill"			: "#1F54A9", // Bleu
				"stroke"		: "none",
				"stroke-width"	: 4,
				"type"			: 1,//forme de la flèche
				
				//"marker-height": 3,
				//"marker-width": 3,
				"z-index"		: 6
				/*"refX"		: 2,
				"refY"			: 5,
				"viewBox"		: "0 0 10 10",
				"markerUnits"	: "strokeWidth",
				"orient"		: "auto",
				"polyline" : {
					"points" 		: "0,0 0,10 5,5 0,0",
					"fill"			: "rgb(31,84,169)",
					"stroke"		: "none",
					"stroke-width"	: "1",
					"marker-height"	: "3",
					"marker-width"	: "3",
					"style"	: {
						"z-index" : 6
					}
				}
				*/
			};
		} else if(value == "Triangle_2"){
			styles = {
				"id"			: "Triangle_2",//obligatoire
				"fill"			: "#FFEA01", // Jaune
				"stroke"		: "none",
				"stroke-width"	: 1,
				"z-index"		: 5,
				"type"			: 2 //forme de la flèche
				/*"refX"	: 2,
				//"refY"	: 5,
				//"viewBox"		: "0 0 10 10",
				//"markerUnits"	: "strokeWidth",
				//"orient"		: "auto",
				//"polyline" : {
					"points" 		: "0,0 0,10 5,5 0,0",
					"fill"			: "rgb(255,234,1)",
					"stroke"		: "rgb(255,234,1)",
					"stroke-width"	: "1",
					"marker-height"	: "5",
					"marker-width"	: "5",
					"style"	: {
						"z-index" : 7
					}
				}*/
			};
		} else if(value == "Triangle_3"){
			styles = {
				"id"			: "Triangle_3",
				"fill"			: "none",
				"stroke"		: "#0000DD",
				"stroke-width"	: "1",
				"type"			: 3
				/*"refX"	: 2,
				"refY"	: 5,
				"viewBox"		: "0 0 10 10",
				"markerUnits"	: "strokeWidth",
				"orient"		: "auto",
				"polyline" : {
					"points" 		: "0,2 8,5 0,8",
					"fill"			: "none",
					"stroke"		: "#0000DD",
					"stroke-width"	: "1"
				}*/
			};
		}
		return styles;
	},
	/**
	 * Créer le support pour les panneaux, puces, etc.
	 */
	_createHtmlContainer : function(width, height){
		this._infoContainer = document.createElement("div");
		this._infoContainer.className = "infoContainer";
		this._infoContainer.style.width = parseInt(width)+"px";
		this._infoContainer.style.height = parseInt(height)+"px";
		
		this._container.appendChild(this._infoContainer);
	},
	
	/**
	 * @deprecated
	 * 
	 * 
	 */
	convertToClass : function(item) {
		/*var s_style = 'iti_default';
		acl = item.split('_');
		if (item == 'iti_adjacent'||acl[1] == 'instruction') s_style = item;
		else if (acl[2] == '10') s_style = 'route_rue';
		else if (acl[2] == '9'||acl[2] == '10'||acl[2] == '11'||acl[2] == '12') s_style = 'route_rue';
		else if (acl[2] == '13'||acl[2] == '14'||acl[2] == '15'||acl[2] == '16') s_style = 'route_principale';
		else if (item == 'iti_6_20'||item == 'iti_6_18') s_style = 'autoroute';
		else if (acl[2] == '20'||acl[2] == '18') s_style = 'type_auto';
		return s_style;*/
	},
	loadDivPanneaux : function(){
		this._divPanneaux = document.createElement('div');
		this._divPanneaux.setAttribute('id','panneaux_'+(iteration+1));
		this._divPanneaux.setAttribute('class','div_panneaux '+pg_fdr_params["type_mep"]);
		this._divPanneaux.style.position = 'absolute';
		this._divPanneaux.style.top = (div_carte.style.top.split('px')[0]*1+5)+'px';
		this._divPanneaux.style.top = div_carte.style.top;
//		this._divPanneaux.style.left = div_carte.style.left;
		this._divPanneaux.style.width = div_carte.style.width;
		this._divPanneaux.style.height = div_carte.style.height;
		if (pg_fdr_params["width_map"] == 0)
		{
			this._divPanneaux.style.display = "none";
		}

		pg_fdr.appendChild(this._divPanneaux);

		$('<div />').addClass('svg_container')
								.appendTo(this._divPanneaux)
								.width($('.div_panneaux:last').width())
								.height($('.div_panneaux:last').height());
		$('<div />').addClass('html_container')
								.appendTo(this._divPanneaux)
								.width($('.div_panneaux:last').width())
								.height($('.div_panneaux:last').height());
		
	},
	
	/**
	 * Retourne le(s) classe en fonction du type de route
	 * @param {String} style
	 * @return {Object}
	 */
	getCssClass : function(type,roadType,importance,level,isoCountry,minMaj) {
		var classes = "";
		
		//ADJACENT
		if (type=="adjacent") {
			if (roadType == 6 && (importance == 20 || importance==18)) {
				classes = "iti_adjacent_autoroute_"+level;
			}
			else if(importance == 20 || importance==18) {
				classes = "iti_adjacent_type_auto_"+level;
			}
			else {//route adjacente par défaut
				classes = "iti_adjacent_"+level;	
			}
		}
		
		//ITINERARY
		if (type=="itinerary") {
			if (importance == 9 || importance == 10 || importance == 11 || importance == 12) {
				classes = 'route_rue_'+level;
			}
			else if (importance == 13 || importance == 14 || importance == 15 || importance == 16) {
				classes = 'route_principale_'+level;
			}
			else if (roadType == 6 && (importance == 20 || importance==18)) {
				classes = "autoroute_"+level;
			}
			else if(importance == 20 || importance==18) {
				classes = "type_auto_"+level;
			}
			else{
				classes = 'iti_default_'+level;
			}
		}
		
		//INSTRUCTIONS
		if (type=="instructions") {
			minMaj = minMaj == 0 ? "mineure" : "majeure";
			classes = 'iti_instruction_' + level + ' iti_instruction_' + level + '_' + minMaj;
		}
		
		return classes;
	},
	
	/**
	 * Convertie le jeu de coordonnées en fonction de l'orientation de l'itinéraire
	 * @param {Array} obj [ [ x, y ], [ x, y ], [ ... ] ]
	 * @param {String} orientation
	 * @return {Array}
	 */
	_flipCoords : function(obj){
		var width = this._width;
		var height = this._height;
		
		if (this._orientation == "C") {
			var newObj = new Array();
			var boxCenter = {x: parseInt(this._width/2), y: parseInt(this._height/2)};
			var list=[], len = obj.length;
			
			// on fait une copie pour ne pas altérer le jeu de coordonnées initial.
			for(var i=0;i<len;i++){
				list[i] = obj[i];
			}
			
			if(list[0] instanceof Array){
				for(var i=0; i<len; i++){
					var item = list[i];
					var x = parseInt(width) - parseInt(item[0]);
					var y = parseInt(height) - parseInt(item[1]);
					
					newObj.push(new Array(x, y));
				}
			} else {
				var x = parseInt(width) - parseInt(list[0]);
				var y = parseInt(height) - parseInt(list[1]);
				newObj = new Array(x, y);
			}
			return newObj;
		}
		return obj;
	},
	/**
	 * Convertie le jeu de coordonnées en fonction de l'orientation de l'itinéraire
	 * @param {Array} points [ [ [ topLeftX, topLeftY, bottomRightX, bottomRightY ] ], [ ... ] ]
	 */
	_flipBox : function(points){
		if (this._orientation=="C"){
			if(points[0] instanceof Array){
				for (var item in points) {
					var point = points[item];
				}
			} else {
				var newCoords = this._flipCoords([ [points[0], points[1]] , [points[2], points[3]] ]);
				points = new Array(newCoords[1][0],newCoords[1][1],newCoords[0][0],newCoords[0][1]); 
			}
		}
		
		return points;
	},
	_flipBox2 : function(fullMapDimension, boxToFlip){
		
		boxResult = boxToFlip;
		if (fullMapDimension.length == 2) fullMapDimension = [0,0].concat(fullMapDimension);
		if (boxToFlip.length == 2)
		{
			boxToFlip = boxToFlip.concat(boxToFlip);
		}

		if (orientation == "C") {
			xFullMapCenter = (fullMapDimension[2] - fullMapDimension[0])/2;
			yFullMapCenter = (fullMapDimension[3] - fullMapDimension[1])/2;
		
			yTranslate = fullMapDimension[3] - (mapDimension[3] + mapDimension[1]);

			// rotation
			
			xMinR = fullMapDimension[0] + fullMapDimension[2] - boxToFlip[2];
			yMinR = fullMapDimension[1] + fullMapDimension[3] - boxToFlip[3];
			xMaxR = fullMapDimension[0] + fullMapDimension[2] - boxToFlip[0];
			yMaxR = fullMapDimension[1] + fullMapDimension[3] - boxToFlip[1];
			
			// translation
			yMinR -= yTranslate;
			yMaxR -= yTranslate;
			
			boxResult = [xMinR,yMinR,xMaxR,yMaxR];
		}
		return boxResult;
	},
	
	/**
	 * Retourne le style en fonction du type (classe) de route
	 * @param {String} classe "classe1 classe2"
	 * @return {Object}
	 */
	getStyle : function(classe){
		var nStyle, classes, nbClasses, i;
		
		nStyle = {};
		
		classe = classe.split(" ");
		nbClasses = classe.length;
		
		for(i = 0; i<nbClasses; i++){
			switch(classe[i]){
				case "sens_unique" :
					
					nStyle["fill"]			= "rgba(255, 255, 255, 0)";
					nStyle["fill-opacity"]	= 0;
					nStyle["stroke"]		= "rgba(255, 255, 255, 0)";
					nStyle["stroke-opacity"] = 0;
					nStyle["stroke-width"]	= 3;
					
					break;
				case "iti_commun" : 
					
					nStyle["fill"] 				= "none";
					nStyle["stroke-linecap"] 	= "round";
					nStyle["stroke-linejoin"] 	= "round";
					
					break;
				case "troncon_extremite" :
					
					nStyle["stroke-linecap"] 	= "butt";
					
					break;
				case "adjacent_road_1" :
					
					nStyle["stroke"] 			= "#AAAAAA";
					nStyle["stroke-width"] 		= 10;
					
					break; 
				case "adjacent_road_2" :
					
					nStyle["stroke"] 			= "#DDDDDD";
					nStyle["stroke-width"] 		= 5;
					       
					break;
				case "iti_default_1" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#A12FA1";
					nStyle["stroke-width"] 		= 6;
					
				case "iti_default_2" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#FF00FF";
					nStyle["stroke-width"] 		= 4;
					       
					break;
				case "iti_adjacent_1" :
					
					nStyle["stroke"] 			= "#AAAAAA";
					nStyle["stroke-width"] 		= 10;
					       
					break;
				case "iti_adjacent_2" :
					
					nStyle["stroke"] 			= "#DDDDDD";
					nStyle["stroke-width"] 		= 5;
					       
					break;
				case "iti_instruction_1" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#1F54A9";
					nStyle["stroke-linecap"] 	= "square";
					nStyle["stroke-linejoin"]	= "miter";
					nStyle["stroke-width"]		= 7;
						
					break;
				case "iti_instruction_2" :
					
					nStyle["stroke"] 			= "#FFEA01";
					nStyle["stroke-linecap"] 	= "square";
					nStyle["stroke-linejoin"]	= "miter";
					nStyle["stroke-width"]		= 4;
						
					break;
				case "iti_instruction_1_majeure" :
					
					nStyle["stroke-width"] 		= 10;
					
					break;
				case "iti_instruction_1_mineure" :
					
					nStyle["stroke-width"] 		= 9;
					
					break;
				case "iti_instruction_2_majeure" :
					
					nStyle["stroke-width"] 		= 5;
					
					break;	
				case "iti_instruction_2_mineure" :
					
					nStyle["stroke-width"] 		= 4;
					
					break;
				case "route_rue_1" :
					
					nStyle["stroke"] 			= "#FF9600";
					nStyle["stroke-width"] 		= 10;
					nStyle["stroke-linecap"] 	= "square";
					nStyle["z-index"] 			= 4;
					       
					break;
				case "route_rue_2" :
					
					nStyle["stroke"] 			= "#FFFFFF";
					nStyle["stroke-width"] 		= 6;
					nStyle["zIndex"] 			= 5;
					       
					break;
				case "route_principale_1" :
					
					nStyle["stroke"] 			= "#FF0000";
					nStyle["stroke-width"] 		= 10;
					nStyle["zIndex"] 			= 4;
					       
					break;
				case "route_principale_2" :
					
					nStyle["stroke"] 			= "#FF9600";
					nStyle["stroke-width"] 		= 6;
					nStyle["zIndex"] 			= 5;
					       
					break;
				case "autoroute_1" :
					
					nStyle["stroke"] 			= "#FF0000";
					nStyle["stroke-width"]	 	= 10;
					nStyle["z-index"] 			= 4;
					
					break;
				case "iti_adjacent_autoroute_1" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#999999";
					nStyle["stroke-width"] 		= 10;
					nStyle["z-index"] 			= 4;
					       
					break;
				case "autoroute_2" :
					
					nStyle["stroke"] 			= "#FFFF00";
					nStyle["stroke-width"] 		= 2;
					nStyle["z-index"] 			= 5;
					
					break;
				case "iti_adjacent_autoroute_2" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#CCCCCC";
					nStyle["stroke-width"] 		= 2;
					nStyle["z-index"] 			= 5;
					       
					break;
				case "type_auto_1" :
					
					nStyle["stroke"] 			= "#FF0000";
					nStyle["stroke-width"]		= 10;
					nStyle["z-index"]			= 4;
					
					break;
				case "iti_adjacent_type_auto_1" :
					
					nStyle["fill"] 				= "none";
					nStyle["stroke"] 			= "#999999";
					nStyle["stroke-width"] 		= 10;
					nStyle["z-index"] 			= 4;
					       
					break;
				case "type_auto_2" :
					
					nStyle["stroke"] 			= "#FFFFFF";
					nStyle["stroke-width"]		= 2;
					nStyle["z-index"]			= 5;
					
					break;
				case "iti_adjacent_type_auto_2" :
					
					nStyle["fill"]				= "none";
					nStyle["stroke"]			= "#FFFFFF";
					nStyle["stroke-width"]	 	= 2;
					nStyle["z-index"]	 		= 5;
					       
					break;
			}
		}
		return nStyle;
	},
	/**
	 * retourne un tronçon partant du premier point d'un tronçon passé en paramètre jusqu'à son centre
	 * Utilisé pour dessiner les flêches de sens des tronçons adjacents
	 */
	_zeroTo50pctPolyline : function(lst_pts, sens) {
		if (sens == -1) lst_pts.reverse();
		
		var distance_totale = 0;
		for (p=1;p<lst_pts.length;p++) {
			distance_totale+= this._distance(lst_pts[p-1],lst_pts[p]);
		}
		var mi_distance = distance_totale / 2;
		var cumul_distance_suivant = 0;
		var cumul_distance_precedent = 0;
		var result = [[lst_pts[0][0],lst_pts[0][1]]];
		var previous_x = lst_pts[0][0];
		var previous_y = lst_pts[0][1];
		
		for (p=1;p<lst_pts.length;p++) {
			var distance_segment = this._distance(lst_pts[p-1],lst_pts[p]);
			cumul_distance_suivant+=distance_segment;
			
			if (cumul_distance_suivant>= mi_distance && cumul_distance_precedent <= mi_distance) {
				var pct_intervalle_utile = (0.50 - cumul_distance_precedent/distance_totale)/((cumul_distance_suivant - cumul_distance_precedent)/distance_totale);
				var x_prec = lst_pts[p-1][0];
				var y_prec = lst_pts[p-1][1];
				var x_suiv = lst_pts[p][0];
				var y_suiv = lst_pts[p][1];
				var x_fin = Math.round(x_prec + ((x_suiv - x_prec)* pct_intervalle_utile));
				var y_fin = Math.round(y_prec + ((y_suiv - y_prec)* pct_intervalle_utile));
				result.push([x_fin,y_fin]);
				break;
			}
			result.push([lst_pts[p][0],lst_pts[p][1]]);
			cumul_distance_precedent+=distance_segment;
		}
		return result;
	},
	/**
	 * 
	 */
	_distance : function (p1,p2) {
		return Math.sqrt(Math.pow(p2[0]-p1[0],2) + Math.pow(p2[1]-p1[1],2));
	},
	drawPucesV1 : function(value, customItems){
		var len = value.length;
		for(var i=0; i<len; i++){
			var checkpoint = value[i], details = value[i].details, j, lenDetails = details.length;
			
			if ($isDefined(checkpoint.schema) && checkpoint.schema.length != 0){
				if (checkpoint.type=="C"){
					// Cas d'un flag
					if (checkpoint.picto=="start"||checkpoint.picto=="step"||checkpoint.picto=="end") { //(isNaN(checkpoint.number)||checkpoint.number=="") {
						this.drawFlag(checkpoint.picto, checkpoint.schema);
					} else if (isNaN(checkpoint.number)||checkpoint.number==""){
						
					} else {
						this.drawIndication("pastille_"+ checkpoint.number, checkpoint.schema);
					}
				} else if (checkpoint.type == "rnu"){
					this.drawRoadLabel(checkpoint.schema, checkpoint.road[0], checkpoint.countryISO, checkpoint.road[1] );
				} else if (checkpoint.type == "fra") {
					this.drawCrossing(checkpoint.model, checkpoint.schema);
				} else if (checkpoint.type == "c"){
					this.drawProximity(checkpoint.city, checkpoint.schema);
				} else {
					var type = checkpoint.type;
					
					if (isNaN(checkpoint.number) || checkpoint.number==""){
						if (type=="rad") type = checkpoint.type +"_"+ checkpoint.speed;
						if (type=="vir") type = checkpoint.type +"_"+ checkpoint.num;
					} else {
						type = "pastille_"+ checkpoint.number;
					}
					this.drawIndication(type, checkpoint.schema);
				}
			}
			
			for (j=0; j<lenDetails; j++){
				var detail = details[j], type = detail.type;
				if($isDefined(detail.schema) && detail.schema.length != 0){
					if (type=="rad") {
						type = detail.type +"_"+ detail.speed;
						this.drawIndication(type, detail.schema);
					} else if (type=="vir") {
						type = detail.type +"_"+ detail.num;
						this.drawIndication(type, detail.schema);
					} else if (type=="rnu") {
						this.drawRoadLabel(detail.schema, detail.road[0], detail.countryISO, detail.road[1] );
					} else if (type=="fra") {
						this.drawCrossing(detail.model, detail.schema);
					} else if (type=="c"){
						this.drawProximity(detail.city, detail.schema);
					} else {
						this.drawIndication(type, detail.schema);
					}
				}
			}
		}
	}
};