JSEPackage("ViaMichelin.Api.Util.HTML");

JSEImportApi("JSE.HTML");

ViaMichelin.Api.Util.HTML = {
	
	/** 
	 * Retourne desc contenant au max size caracteres suivis de '..'
	 * @param {String} desc
	 * @param {int} size
	 * @param {String} character
	 * @return {String}
	 */
	truncDescription : function(desc, size, character) {
		if(character == null || character == ""){
			character = "..";
		}
		if (!desc) return "";
		if (desc.length <= size) return desc;
		//else return desc.substring(0, Math.max(desc.lastIndexOf(" ", size), desc.lastIndexOf("-", size))) + "..";
		else {
			//On supprime des éventuels &#xxx; qui seraient tronqués en fin de desc
			//Si on a un & présent qui n'est pas suivi par un ; et qui est dans les 6 derniers caractères de la chaine
			var newDesc = desc.substring(0, size);
			var indexOfEtCom = newDesc.lastIndexOf("&");
			var indexOfPointVirgule = newDesc.lastIndexOf(";");
			if (indexOfEtCom != -1 && indexOfPointVirgule < indexOfEtCom && indexOfEtCom > Math.max(0, size - 6)) {
				newDesc = newDesc.substring(0, newDesc.lastIndexOf("&"));
			}
			return newDesc + character;
		}
	},
	/**
	 * Adapte le contenu "texte" d'un element HTML, pour éviter les retour à la ligne.
	 * Rajoute ".." à la fin si le texte est tronqué.
	 * 
	 * Pré-requis : 
	 * - seulement la "width" et la "line-height" doivent être renseigné sur ce champs.
	 * 
	 * Indication pour éviter des problèmes : 
	 * - Un texte de font-size :12px à une taille (height) à 15px => donc la "line-height" doit être à 15px
	 * - Un texte de font-size :11px à une taille (height) à 14px => donc la "line-height" doit être à 14px
	 * 
	 * @param {HTMLElement} element
	 * @param {String} character caractère de fin (par défaut "..")
	 * @return {void}
	 */
	adaptableDescription : function(element, character) {
		if(typeof(element.innerHTML) == "string" && element.innerHTML != "" && element.innerHTML != null){
			
			var eParent = element;
			var gap = 0;
			
			// Cas d'un <a></a>
			if(element.href != null){
				eParent = element.parentNode;
			}
			
			//Calcul gap
			var marginTop     = parseInt(JSE.HTML.getStyle(eParent, "marginTop"));
			var marginBottom  = parseInt(JSE.HTML.getStyle(eParent, "marginBottom"));
			var paddingTop    = parseInt(JSE.HTML.getStyle(eParent, "paddingTop"));
			var paddingBottom = parseInt(JSE.HTML.getStyle(eParent, "paddingBottom"));
			
			gap = parseInt(marginTop + marginBottom + paddingTop + paddingBottom);
			
			var characters = element.innerHTML;
			var size = JSE.HTML.getStyle(eParent, "lineHeight");
			
			if(size){
				size = parseInt(size.replace("px", "").replace("pt", "").replace("e", ""));
			}
			
			for(var i=characters.length; i>0; i--){
				var truncD = ViaMichelin.Api.Util.HTML.truncDescription(characters, i, character);
				element.innerHTML = truncD;
				
				if(parseInt(eParent.offsetHeight) <= (size + gap)){
					break;
				}
			}
			// Bridage : pour être sur que le texte ne dépassera pas
			eParent.style.overflow = "hidden";
			if(size) eParent.style.height = size+"px";
			element.className = element.className.replace(/adaptable/gi, "adapted");
		}
	},
	
	/**
	 * Supprime les tags html ex: <img src="xxx" /> du texte
	 * @param {String} text
	 * @return {String}
	 */
	removeTags : function(text){
		var reg = new RegExp("<.[^<>]*>", "gi" );
		return text.replace(reg, "");
	},
	
 	/**
 	 * Ajoute une classe à un élément HTMLElement
 	 * @param {HTMLElement} element
 	 * @param {String} className
 	 * @return {void}
 	 */
 	addClassNameToHTMLElement : function(element, className){
 		if(!ViaMichelin.Api.Util.HTML.isClassNameExistInHTMLElement(element, className)){
			element.className = element.className+" "+className;
		}
 	},
 	/**
 	 * Supprime une classe d'un élément HTMLElement
 	 * @param {HTMLElement} element
 	 * @param {String} className
 	 * @return {void}
 	 */
 	removeClassNameToHTMLElement : function(element, className){
 		if(className instanceof Array){
 			for(var item in className){
 				if($isDefined(className[item])){
 					element.className = element.className.replace(className[item], "");
 				}
 			}
 		} else {
 			element.className = element.className.replace(className, "");
 		}
 	},
 	/**
 	 * Indique si un className est présent au sein des className d'un HTMLElement
 	 * @param {HTMLElement} element
 	 * @param {String} clsName
 	 * @return {boolean}
 	 */
 	isClassNameExistInHTMLElement : function(element, clsName){
 		var classNames = element.className.split(" ");
 		for(var i=0; i<classNames.length; i++){
 			if(classNames[i] == clsName){
 				return true;
 			}
 		}
 		return false;
 	},
 	/**
 	 * Recherche des élements du DOM ayant pour className : clsName
 	 * @param {HTMLElement} element
 	 * @param {String} clsName
 	 * return {Array}
 	 */
 	getElementsByClassName : function(element, clsName){
 		var retVal = new Array();
 		if(typeof(element) != "undefined" && element != null){
			
			if (element.getElementsByClassName) {
				var elements = element.getElementsByClassName(clsName),
					len = elements.length;
				for (var i=0; i<len; i++) {
					retVal.push(elements[i]);
				}
				return retVal;
			}
			
			var elements = element.getElementsByTagName("*"),
				len = elements.length;
			for (var i=0; i<len; i++) {
				if (elements[i].className.indexOf(" ") >= 0) {
					var classes = elements[i].className.split(" ");
					for (var j = 0;j < classes.length;j++) {
						if (classes[j] == clsName)
							retVal.push(elements[i]);
					}
				}
				else if (elements[i].className == clsName)
					retVal.push(elements[i]);
			}
 		}
 		return retVal;
	},
	/**
	 * Vide les element HTML d'un noeud
	 * @return {void}
	 */
	removeHTMLList: function(element){
		if (element != null) {
			while(element.childNodes.length > 0){
				element.removeChild(element.childNodes[0]);
			}
		}
	},
	/**
	 * Met au pluriel un nom si besoin.
	 * @param {String} count le nombre d'occurence
	 * @param {String} singular le nom au singulier
	 * @param {String} plural (optionnal) le pluriel du nom (si non-régulier)
	 * @return {String}
	 */
	pluralize: function(count, singular, plural){
		if(typeof(count) != "number") count = 0;
		if(typeof(plural) == "undefined" || plural == null || plural == "") plural = null;
		
  		if(count <= 1) { 
  			return singular;
  		} else if(plural!=null) {
  			return plural;
  		} else {
  			return singular+ "s";
  		}
	},
	/**
	 * Retourne le mot avec la premiere lettre en majuscule
	 * @param {String} element
	 * @return {String}
	 */
	capitalize : function(element){
		/*
		return element.replace(/\w+/g, function(a){
	        return a.charAt(0).toUpperCase() + a.substr(1).toLowerCase();
	    });
	    */
	    return element.replace(/[^\s-]+/g, function (a) {
	    	return a.charAt(0).toUpperCase() + a.substr(1).toLowerCase();
	    });
	},
	/**
	 * Décode dans une chaîne de caractères les caractères spéciaux encodé en HTML.
	 * Elle est le pendant de la méthode java HtmlCleaner.htmlFilter(value) 
	 * [ou DefaultData.htmlFilter(value)].
	 * Elle décode les séquences "&lt;", "&gt;", "&quot;", "&#39;", "&amp;", "<br>", "<br/>".
	 */
	htmlDecode : function(value) {
		if (value) {
			value = value.replace(/&lt;/g, '<');
			value = value.replace(/&gt;/g, '>');
			value = value.replace(/&quot;/g, '\"');
			value = value.replace(/&#39;/g, '\'');
			value = value.replace(/&amp;/g, '&');
			value = value.replace(/<br>/g, '\n');
			value = value.replace(/<br\/>/g, '\n');
		}
		return value;
	},
	/**
	 * Teste si l'item existe dans la liste.
	 * @param {GenericItem} item
	 */
	isItemExistInList : function(list, item){
		for(var i=0;i<list.length;i++){
			if(item == list[i]){
				return true;
			}
		}
		return false;
	},
	/**
	 * @param {HTMLElement} field
	 * @return
	 */
	focused : function(field){
		
		/* exp car expando est utilisé par la completion */
		field.exp = this;
		field.onfocus = field.onkeypress = function () {
			this.exp._focusFieldWithDefVal(this);
		};
		field.onblur = function () {
			this.exp._blurFieldWithDefVal(this);
		};
		if (field.value == "") {
			this._blurFieldWithDefVal(field);
			field.blur();
		}
	},
	/**
	 * Supprime la valeur par défaut d'un champ lorsqu'on met le focus dessus.
	 * @param field
	 * @return {void}
	 */
	_focusFieldWithDefVal: function (field) {
		if (field.className.indexOf("fFocused") < 0) {
			field.className += " fFocused";
		}
	},
	/**
	 * Remet la valeur par défaut d'un champ lorsqu'on perd le focus.
	 * @param field
	 * @return {void}
	 */
	_blurFieldWithDefVal: function (field) {
		if (field.value == "") {
			field.className = field.className.replaceAll(" fFocused", "");
		}
	},
	/**
 	 * Ajoute l'element au tableau s'il n'est pas présent sinon l'enlève
 	 * @param {Array} list
 	 * @param {Object} value
 	 * @return {Array}
 	 */
 	addOrRemoveValueInArray : function(list, value){
 		var nList = new Array();
 		for(var i=0; i<list.length ;i++){
 			if(list[i] != value) nList.push(list[i]);
 		}
 		return nList;
 	},
 	/**
	 * Fonction pour detecter si le navigateur gère le SVG
	 */
	isSupportSVG : function(){
		return !!document.createElementNS && !!document.createElementNS('http://www.w3.org/2000/svg' , "svg").createSVGRect;
	},
	
	/**
	 * Convertit une chaîne xml en objet DOM
	 * @param  {String} xmlString La chaîne XML à transformer
	 * @return {Object} L'objet DOM
	 */
	getDomObjectFromXMLString : function (xmlString) {
		// on transforme les balises orphelines en ouvrante/fermante
		xmlString = xmlString.replace(/<([a-zA-z]*)([^\/>]*)\/>/g, "<$1$2></$1>");
		
		if (window.DOMParser) {
			var parser = new DOMParser();
			domObject = parser.parseFromString(xmlString, 'text/xml');
		}
		else // Internet Explorer
		{
			domObject = new ActiveXObject("Microsoft.XMLDOM");
			domObject.async = false;
			domObject.loadXML(xmlString); 
		}
		return domObject;
	}
};

ViaMichelin.Api.Util.extend(ViaMichelin.Api.Util.HTML);