JSEPackage("ViaMichelin.Api.Util.Other");

JSEImportApi("ViaMichelin.Api.Model.MUPPoi");
JSEImportApi("ViaMichelin.Api.Model.MPMPoi");
JSEImportApi("ViaMichelin.Api.Model.Destination");
JSEImportApi("ViaMichelin.Api.Constants.MyVM");
JSEImportApi("ViaMichelin.Api.Constants.MPMPoi");

ViaMichelin.Api.Util.Other = {
	/**
	 * Ajoute une chaine de caractères à une autre si celle-ci n'est pas présente
	 * et retourne le résultat.
	 * @param {String} element
	 * @param {String} text
	 * @param {String} separator
	 * @return {String}
	 */
	addString : function(element, text, separator){
		if(typeof(element)=="undefined" || element==null){
			element = "";
		}
		if(typeof(separator)=="undefined" || separator==null){
			separator = " ";
		}
		if(!ViaMichelin.Api.Util.Other.isStringExist(element, text, separator)){
			element = element+separator+text;
		}
		return element;
	},
	/**
 	 * Indique si une chaine de caractères est présente dans la chaine de caractères source.
 	 * @param {String} element
 	 * @param {String} text
 	 * @param {String} separator
 	 * @return {boolean}
 	 */
 	isStringExist : function(element, text, separator){
 		if(typeof(separator)=="undefined" || separator==null){
			separator = " ";
		}
 		element = separator+ element +separator;
 		text    = separator+ text +separator;
 		
 		if(element.indexOf(text) == -1)
 			return false;
 		else 
 			return true;
 	},
 	
	/**
	 * Teste l'existence d'un élément dans un tableau
	 * @param  {Array}   list  Le tableau sur lequel porte la recherche
	 * @param  {mixed}   value La donnée à rechercher
	 * @return {Boolean} true si la donnée est présente dans le tableau
	 */
	existsInArray : function (list, value) {
		for (var i = 0; i < list.length; i++) {
			if (list[i] === value) return true;
		}
		return false;
	},
 	
	// teste la chaine de caracteres
	testPropValue : function(propriety) {
		return ((propriety != "") && (propriety != " ") && (typeof(propriety) != "undefined") && (propriety != "0m"));
	},

	/**
	 * fonction de conversion m -> km
	 * @param {float} n
	 * @param {float} d
	 */ 
	closer : function(n,d) {
		var dNeg = n%d;
		var dPos = d - dNeg;
		return (dPos <= dNeg) ? (dPos+n) : (n-dNeg);
	},
	/**
	 * Méthode utilisée pour calculer les valeurs des longueurs (en km ou mi) à afficher dans la feuille de route.
	 * @param {int} i longueur (en m ou mi)
	 * @param {String} u : km ou mi
	 */
	makeDist : function(i, u){
		if (!$isDefined(i)) return "";
		
		//on force i en entier
		i = parseInt(i);
		/* On ajoute un espace insécable à l'unité de distance */
		var unit;
		if(u!=null && !(u.indexOf("&nbsp;")>-1)) {
			unit = "&nbsp;" + u;
		} else {
			unit = u;
		}
		if (i == 0 || typeof i == 'undefined') return "";
		
		if (u.toLowerCase() == "mi") {
			i = i*1000;
			if (i < 100) return "<0.1" + unit;
			if (i < 2001) return (this.closer(i,100)/1000 + unit);
			if (i < 10001) return (this.closer(i,500)/1000 + unit);
			return (this.closer(i,1000)/1000 + unit);
		} else {
			if (i < 100) return "<0.1"+unit;
			if (i < 2001) return ((Math.round(this.closer(i,100)/100)/10) + unit);
			if (i < 10001) return ((Math.round(this.closer(i,500)/100)/10) + unit);
			return (Math.round(this.closer(i,1000)/1000) + unit);
		}
	},	
	/**
	 * Formate une horaire.
	 * @param {Number} totalTime Temps total en secondes.
	 * @return String
	 */	
	setISO8601 : function (string) {
	    var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
	        "(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?" +
	        "(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
	    var d = string.match(new RegExp(regexp));
	    var offset = 0;
	    var date = new Date(d[1], 0, 1);
	    if (d[3]) { date.setMonth(d[3] - 1); }
	    if (d[5]) { date.setDate(d[5]); }
	    if (d[7]) { date.setHours(d[7]); }
	    if (d[8]) { date.setMinutes(d[8]); }
	    if (d[10]) { date.setSeconds(d[10]); }
	    if (d[12]) { date.setMilliseconds(Number("0." + d[12]) * 1000); }
	    if (d[14]) {
	        offset = (Number(d[16]) * 60) + Number(d[17]);
	        offset *= ((d[15] == '-') ? 1 : -1);
	    }
	    offset -= date.getTimezoneOffset();
	   return (Number(date) + (offset * 60 * 1000));
	},
	/**
	 * Formate une horaire.
	 * @param {Number} totalTime Temps total en secondes.
	 * @return String
	 */
	makeHour: function(totalTime) {
		if (totalTime == 0) {
			if (ViaMichelin.Api.Constants.System.ApiLang == 'nld') {
				return "00.00&nbsp;u";
			}
			else {
				return "00h00";
			}
		}
		var seconds = totalTime;
		var minutes = 0;
		var hours = 0;

		while (seconds >= 60) {
			minutes += 1;
			seconds -= 60;
		}
		while (minutes >= 60) {
			hours += 1;
			minutes -= 60;
		}
		//Cas spécifique au néerlandais : 04.11 u, pour les autres langues : 04h11
		if (ViaMichelin.Api.Constants.System.ApiLang == 'nld')
			return (((hours<10) ? "0"+hours : hours) + "." + ((minutes<10)?"0"+minutes:minutes) + "&nbsp;u");
		else
			return (((hours<10) ? "0"+hours : hours) + "h" + ((minutes<10)?"0"+minutes:minutes));
	},
	// rajoute le zero avant les unites pour toujours avoir la bonne longueur de chaine de caracteres
	formatDateNumber : function(num){
		return ((num < 10) ? "0" + num : num);
	},
	/**
	 * Formatte une date en fontion d'un pattern
	 * Elements acceptés : d m Y h i s
	 * @return String;
	 */
	dateFormat : function(format, date) {
		if (date == undefined) {
			date = new Date();
		}
		if (typeof date == 'number') {
			time = new Date();
			time.setTime(date);
			date = time;
		} else if (typeof date == 'string') {
			date = new Date(date);
		}
		var fullYear = date.getYear();
		if (fullYear < 1000) {
			fullYear = fullYear + 1900;
		}
		var hour = date.getHours();
		var day = date.getDate();
		var month = date.getMonth() + 1;
		var minute = date.getMinutes();
		var seconde = date.getSeconds();
		var milliSeconde = date.getMilliseconds();
		var reg = new RegExp('(d|m|Y|H|i|s)', 'g');
		var replacement = new Array();
		replacement['d'] = day < 10 ? '0' + day : day;
		replacement['m'] = month < 10 ? '0' + month : month;
		replacement['Y'] = fullYear;
		replacement['Y'] = fullYear;
		replacement['H'] = hour < 10 ? '0' + hour : hour;
		replacement['i'] = minute < 10 ? '0' + minute : minute;
		replacement['s'] = seconde < 10 ? '0' + seconde : seconde;
		return format.replace(reg, function($0) {
			return ($0 in replacement) ? replacement[$0] : $0.slice(1, $0.length - 1);
		});
	},
	trim : function (value){
		return value.replace(/^\s+/,'').replace(/\s+$/,'');
	},
	/**
	 * @param {String} str : chaine à nettoyer
	 * @param {char} chars : caractère à suprimmer
	 */
	rtrim : function rtrim(str, chars) {
		chars = chars || "\\s";
		return str.replace(new RegExp("[" + chars + "]+$", "g"), "");
	},
	/**
	 * @param {String} old_date Date ex: 17/01/2009
	 * @param {int} days Nombre de jour à ajouter
	 */
	addDaysToDate : function (old_date, days){
	    var new_date_text = "";
	    
	    if(old_date != "" && old_date != null) {
		    // Date plus plus quelques jours
		    var split_date = old_date.split('/');
		    // Les mois vont de 0 a 11 donc on enleve 1, cast avec *1
		    var new_date = new Date(split_date[2], split_date[1]*1 - 1, split_date[0]*1 + days);
		    var new_day = new_date.getDate();
		        new_day = ((new_day < 10) ? '0' : '') + new_day; // ajoute un zéro devant pour la forme  
		    var new_month = new_date.getMonth() + 1;
		        new_month = ((new_month < 10) ? '0' : '') + new_month; // ajoute un zéro devant pour la forme  
		    var new_year = new_date.getYear();
		        new_year = ((new_year < 200) ? 1900 : 0) + new_year; // necessaire car IE et FF retourne pas la meme chose  
		    new_date_text = new_day + '/' + new_month + '/' + new_year;
	    }
	    return new_date_text;
 	},
 	/**
 	 * Détection d'IE6 pour les css
 	 * @return {boolean}
 	 */
 	isIE6CSS : function(){
 		if(ViaMichelin.Api.Util.Other.getVersionIECSS() == 6){
 			return true;
 		} else {
 			return false;
 		}
 	},
 	/**
 	 * Détection d'IE7 pour les css
 	 * @return {boolean}
 	 */
 	isIE7CSS : function(){
 		if(ViaMichelin.Api.Util.Other.getVersionIECSS() == 7){
 			return true;
 		} else {
 			return false;
 		}
 	},
 	/**
 	 * Détection d'IE8 pour les css
 	 * @return {boolean}
 	 */
 	isIE8CSS : function(){
 		if(ViaMichelin.Api.Util.Other.getVersionIECSS() == 8){
 			return true;
 		} else {
 			return false;
 		}
 	},
 	getVersionIECSS : function(){
 		if ((window.ActiveXObject && !window.opera) && (/MSIE (\d+\.\d+);/.test(navigator.userAgent))) {
			var ieversion = new Number(RegExp.$1);
			if (ieversion>=8){
				return 8;
			} else if (ieversion>=7) {
				return 7;
			} else if (ieversion>=6) {
				return 6;
			}
		}
		return 0;
 	},
 	/**
 	 * Teste l'existence de parametres dans l'url
 	 * Exemple : list = ["c1","c2","c3"]
 	 * Si un element de list est présent dans l'url on retourne true
 	 * @param {Array} list liste des paramettres
 	 * @return {Boolean}
 	 */
 	 isParametersExist : function(list){
 	 	try{
	 	 	if(list.length > 0){
	 	 		if(window.location.href != ""){
		 	 		var lineParams = window.location.href.split("?");
		 	 		params = lineParams[1].split("&");
		 	 		
		 	 		for(var i=0; i<params.length; i++){
		 	 			var param = params[i].split("=");
		 	 			
		 	 			for(var item in list){
		 	 				if(list[item] == param[0]){
		 	 					return true;
		 	 				}
		 	 			}
		 	 		}
	 	 		}
	 	 	}
 	 	}catch(e){}
 	 	return false;
	},
	/**
	 * Génère les urls des css en fonction du nom et de la version
	 * @param {String} name
 	 * @param {String} version
	 * @param {Object} cssNavigator
	 * @return
	 */
	generateUrlCss : function(name, version, cssNavigator){
		var urls = new Array();
 	 	
 	 	var cssName = "";
 	 	if($isDefined(version) && version!=""){
 	 		cssName = name+"V"+version+".css";
 	 	}else{
 	 		cssName = name+".css";
 	 	}
 	 	var url = ViaMichelin.Api.Constants.System.ApiCssRoot + "all/"+ cssName;
 	 	urls.push(url);
 	 	
 	 	if(typeof(cssNavigator)!="undefined" && cssNavigator != null){
	 	 	if(this.getVersionIECSS()!=0 && cssNavigator["ie"]){
	 	 		var url = "";
	 	 		if(ViaMichelin.Api.Util.Other.isHttps()){
	 	 			url = ViaMichelin.Api.Constants.System.ApiCssRoot + "https/ie/"+ cssName;
	 	 		} else {
	 	 			url = ViaMichelin.Api.Constants.System.ApiCssRoot + "ie/"+ cssName;
	 	 		}
	 	 		urls.push(url);
	 	 	}
	 	 	if(this.getVersionIECSS()==6 && cssNavigator.ie6){
	 	 		var url = "";
	 	 		if(ViaMichelin.Api.Util.Other.isHttps()){
	 	 			url = ViaMichelin.Api.Constants.System.ApiCssRoot + "https/ie6/"+ cssName;
	 	 		} else {
	 	 			url = ViaMichelin.Api.Constants.System.ApiCssRoot + "ie6/"+ cssName;
	 	 		}
	 	 		urls.push(url);
	 	 	}
	 	 	if(this.getVersionIECSS()==7 && cssNavigator.ie7){
	 	 		var url = ViaMichelin.Api.Constants.System.ApiCssRoot + "ie7/"+ cssName;
	 	 		urls.push(url);
	 	 	}
	 	 	if(this.getVersionIECSS()==8 && cssNavigator.ie8){
	 	 		var url = ViaMichelin.Api.Constants.System.ApiCssRoot + "ie8/"+ cssName;
	 	 		urls.push(url);
	 	 	}
 	 	}
		return urls;
	},
	loadedCss : [],
	/**
	 * Charge les css contenus dans "urls"
 	 * @param {String} name indique un nom de classe pour savoir si la feuille de style existe déjà
 	 * @param {Array} urls contient les urls des css
 	 * @return
 	 */
	loadCss : function(name, urls, options){
		//Chargement des css
		var cssDefined = JSE.HTML.isDefinedSelector("is"+ name +"Imported");
		// Si on importe trop de CSS, IE finit par nous envoyer balader
		// comme le cssDefined ne marche pas toujours, on blinde avec un cssInArray en plus
		var cssInArray = $isDefined(ViaMichelin.Api.Util.Other.loadedCss[name]);
		if (!cssDefined && !cssInArray){
			for(var i=0; i<urls.length; i++){
				var cssName = urls[i];
				var eLink = document.createElement("link");
				eLink.type = "text/css";
				eLink.rel = "stylesheet";
				eLink.href = cssName;
				
				if(typeof(options) != "undefined"){
					for(var item in options){
						eLink[item] = options[item];
					}
				}
				
				var head = document.getElementsByTagName("head")[0];
				head.appendChild(eLink);
			}
			ViaMichelin.Api.Util.Other.loadedCss[name] = true;
		}
	},
	/**
	 * Extrait les informations d'une image
	 * @param {String} src
	 * @param {Object} callback
	 * @param {Object} instance
	 */
	getInfosImage : function(src, callback, instance){
		try{
			var prop = {
				width  : 0,
				height : 0
			};
			
			var nImg = new Image();
			nImg.onload = function(){
				if (nImg.width>0 && nImg.height>0) {
					prop.size = {
						width : nImg.width,
						height: nImg.height
					}
					
					callback.call(instance, prop);
				}
			};
			nImg.src = src;
		
		}catch(e){
			alert("Exc:"+ e);
		}
	},
	getMousePosition : function(e){
		e = window.event || e;
		var x;
		var y;
		if (navigator.isMozilla) {
			x = e.pageX;
			y = e.pageY;
		} else if (navigator.isSafari) {
			x = e.pageX + document.body.offsetLeft;
			y = e.pageY + document.body.offsetTop;
		} else {
			x = e.clientX + document.documentElement.scrollLeft;
			y = e.clientY + document.documentElement.scrollTop;
		}
		return {
			x : x,
			y : y
		};
	},
	toRGB: function(color){
		var colorR, colorG, colorB;
		var html = color;
		 
		// Parse out the RGB values from the HTML Code
		if (html.substring(0, 1) == "#") {
			html = html.substring(1);
		}
		 
		if (html.length == 3) {
			colorR = html.substring(0, 1);
			colorR = colorR + colorR;
			     
			colorG = html.substring(1, 2);
			colorG = colorG + colorG;
			 
			colorB = html.substring(2, 3);
			colorB = colorB + colorB;
		} else if (html.length == 6) {
			colorR = html.substring(0, 2);
			colorG = html.substring(2, 4);
			colorB = html.substring(4, 6);
		}
		 
		// Convert from Hex (Hexidecimal) to Decimal
		colorR = parseInt(colorR, 16);
		colorG = parseInt(colorG, 16);
		colorB = parseInt(colorB, 16);
		
		return {"r": colorR, "g": colorG, "b": colorB};
	},
	/**
	 * Formatte une phrase avec une ponctuation en fonction de la langue.
	 * @param {String} p 		: la ponctuation à mettre
	 * @param {String} start 	: le début de phrase
	 * @param {String} end 	: la fin de phrase
	 * @param {String} lang 	: code ISO (3 caractères) de la langue 
	 */
	formattingPunctuation : function(p, start, end, lang){
		var buffer = "";
		if(lang.toLowerCase() == "fra")
			buffer+= start+p+" "+end;
		else
			buffer+= start+" "+p+" "+end;
			
		return buffer;
	},
	/**
	 * @param {float} dist Distance en metre
	 */
	formattingDistance : function(dist){
		if(dist <= 1000){
			return "1 km";
		} else if(dist > 1000){
			return parseInt(dist/1000)+" km";
		}
		return "";
	},
	/**
	 * Serialize un objet
	 * @param {Object} obj
	 */
	serialize : function(obj){
		var returnVal;
		if(obj != undefined){
			switch(obj.constructor) {
				case Array:
					var vArr="[";
					for(var i=0;i<obj.length;i++) {
						if(i>0) vArr += ",";
							vArr += ViaMichelin.Api.Util.Other.serialize(obj[i]);
					}
					vArr += "]"
					return vArr;
				case String:
					returnVal = escape("'" + obj + "'");
					return returnVal;
				case Number:
					returnVal = isFinite(obj) ? obj.toString() : null;
					return returnVal;    
				case Date:
					returnVal = "#" + obj + "#";
					return returnVal;  
				default:
					if(typeof obj == "object"){
						var vobj=[];
						for(attr in obj) {
							if(typeof obj[attr] != "function") {
								vobj.push('"' + attr + '":' + ViaMichelin.Api.Util.Other.serialize(obj[attr]));
							}
						}
						if(vobj.length >0)
							return "{" + vobj.join(",") + "}";
						else
							return "{}";
					} else {
						return obj.toString();
				}
			}
		}
		return null;
	},
	/**
	 * Adapte le protocole utilisé dans l'url en fonction du protocole du site.
	 * Rajoute le parametre "protocol=https" pour utiliser l'apirest en mode https 
	 * @param {String} url
	 * @return
	 */
	protocolAdapter :function(url){
		if (!url || typeof url != "string") return url;
		
		if(window.location.protocol.indexOf("https")!=-1 && url.indexOf("https")==-1){
			return (url.replace("http://", "https://"))+"&protocol=https";
		} else if(url.indexOf("https")==0){
			return url+"&protocol=https";
		} else {
			return url;
		}
	},
	/**
	 * Indique si l'on est en https
	 * @return {boolean}
	 */
	isHttps : function(){
		if(window.location.protocol.indexOf("https")!=-1){
			return true;
		} else {
			return false;
		}
	},
	/**
	 * Mixe les attributs entres eux.
	 * Ajoutes les attributs de attr2 à attr1 
	 * @param {Object} attr1
	 * @param {Object} attr2
	 */
	mixAttributes : function(attr1, attr2){
		if($isNotEmpty(attr2)){
			for (var item in attr2){
				if(typeof(attr2[item]) == "object" && !$isDomElement(attr2[item])) {
					if(!$isDefined(attr1[item])){
						attr1[item] = attr2[item];
					// if Array
					} else if(attr2[item] instanceof Array) {//attr2[item].length != "undefined"){//array
						attr1[item] = attr2[item].slice(0);
					} else {
						if (!$isNotEmpty(attr1[item])) {
							attr1[item] = {};
						}
							
						for(var ele in attr2[item]) {
							attr1[item][ele] = attr2[item][ele];
						}
					}
				} else {
					attr1[item] = attr2[item];
				}
			}
		}
	},
	/**
	 * Retourne l'univers par defaut de la resource
	 * @param {Object} resource
	 */
	getDefaultOrigin : function(resource){
		var universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE["V"+"M"];
		var device = ViaMichelin.Api.Constants.MyVM.ORIGIN.DEVICE.BROWSER;
		
		if (resource instanceof ViaMichelin.Api.Model.MUPPoi || resource instanceof ViaMichelin.Api.Model.MPMPoi){
			var type = resource.getType();
			switch(type){
				case ViaMichelin.Api.Constants.MPMPoi.RESTAURANT :
					universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE.GM21;
				break;
				case ViaMichelin.Api.Constants.MPMPoi.HOTEL :
					universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE["V"+"M"];
				break;
				case ViaMichelin.Api.Constants.MPMPoi.TOURISM :
				case ViaMichelin.Api.Constants.MPMPoi.LEISURE :
					universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE.VMV;
				break;
			}
		} else if (resource instanceof ViaMichelin.Api.Model.Destination){
			universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE.VMV;
		} else {
			universe = ViaMichelin.Api.Constants.MyVM.ORIGIN.UNIVERSE["V"+"M"];
		}
		
		return { "universe": universe, "device": device};
	},
	/**
	 * Affiche les commentaires pour le mode choisi
	 * @param {String} template
	 * @param {String} mode
	 * @return {String}
	 */
	templateProcess : function(template, mode) {
		if (!$isDefined(template)) return "";
		if (!$isDefined(mode)) mode = "DEFAULT";
		
		var pos = template.indexOf("##START_", 0);
		var tempo = 0;
		while(pos!=-1){
			// Test du mode
			if (template.indexOf("##START_"+mode+"##", pos) == pos) {
				// c'est le mode
				template = template.replace("##START_"+mode+"##", "");
				template = template.replace("##END_"+mode+"##", "");
			} else {
				// ce n'est pas le mode : suppression du code
				var end = template.indexOf("##", pos+1);
				var nmode = template.substring(pos+8, end);
				var end = template.indexOf("##END_"+nmode+"##");
				var firstPart = template.substring(0, pos);
				var lastPart = template.substring(end + nmode.length + 8, template.length);
				template = firstPart+lastPart;
			}
			
			pos = template.indexOf("##START_", pos);
		}
		return template;
	}
};

function $isNotEmptyArray(object){
	return ( (object instanceof Array) && object.length>0);
}

ViaMichelin.Api.Util.extend(ViaMichelin.Api.Util.Other);